@@ -20,18 +20,28 @@
 # MA 02110-1301, USA.
 
 package Lintian::binaries;
-use strict;
+
+use v5.20;
 use warnings;
+use utf8;
 use autodie;
 
+use File::Spec;
+use List::MoreUtils qw(any);
+
+use Lintian::Data;
+use Lintian::Relation qw(:constants);
+use Lintian::Spelling qw(check_spelling);
+use Lintian::Util qw(strip);
+
 use constant NUMPY_REGEX => qr/
     \Qmodule compiled against ABI version \E (?:0x)?%x
     \Q but this version of numpy is \E (?:0x)?%x
-/xo;
+/x;
 
 # Guile object files do not objdump/strip correctly, so exclude them
 # from a number of tests. (#918444)
-use constant GUILE_PATH_REGEX => qr,^usr/lib/[^/]+/[^/]+/guile/[^/]+/.+\.go$,o;
+use constant GUILE_PATH_REGEX => qr,^usr/lib/[^/]+/[^/]+/guile/[^/]+/.+\.go$,;
 
 # These are the ones file(1) looks for.  The ".zdebug_info" being the
 # compressed version of .debug_info.
@@ -39,14 +49,12 @@ use constant GUILE_PATH_REGEX => qr,^usr/lib/[^/]+/[^/]+/guile/[^/]+/.+\.go$,o;
 #   not strippable for static libs.  Accordingly, it is omitted below.
 use constant DEBUG_SECTIONS => qw(.debug_info .zdebug_info);
 
-use File::Spec;
-use List::MoreUtils qw(any);
+use constant EMPTY => q{};
 
-use Lintian::Check qw(check_spelling spelling_tag_emitter);
-use Lintian::Data;
-use Lintian::Relation qw(:constants);
-use Lintian::Tags qw(tag);
-use Lintian::Util qw(internal_error strip);
+use Moo;
+use namespace::clean;
+
+with 'Lintian::Check';
 
 my $ARCH_REGEX = Lintian::Data->new('binaries/arch-regex', qr/\s*\~\~/o,
     sub { return qr/$_[1]/ });
@@ -58,6 +66,13 @@ my $BINARY_SPELLING_EXCEPTIONS
 my %PATH_DIRECTORIES = map { $_ => 1 } qw(
   bin/ sbin/ usr/bin/ usr/sbin/ usr/games/ );
 
+sub spelling_tag_emitter {
+    my ($self, @orig_args) = @_;
+    return sub {
+        return $self->tag(@orig_args, @_);
+    };
+}
+
 sub _embedded_libs {
     my ($key, $val, undef) = @_;
     my $result = {'libname' => $key,};
@@ -74,15 +89,15 @@ sub _embedded_libs {
             } elsif ($opt eq 'source-regex') {
                 $result->{$opt} = qr/$val/;
             } else {
-                internal_error("Unknown option $opt used for $key"
-                      . ' (in binaries/embedded-libs)');
+                die
+"Unknown option $opt used for $key (in binaries/embedded-libs)";
             }
         }
     }
 
     if (defined $result->{'source'} and $result->{'source-regex'}) {
-        internal_error("Both source and source-regex used for $key"
-              . ' (in binaries/embedded-libs)');
+        die
+"Both source and source-regex used for $key (in binaries/embedded-libs)";
     } else {
         $result->{'source'} = $key unless defined $result->{'source'};
     }
@@ -114,8 +129,13 @@ our $OBSOLETE_CRYPT_FUNCTIONS
 
 our $ARCH_32_REGEX;
 
-sub always {
-    my ($pkg, $type, $info, $proc, $group) = @_;
+sub installable {
+    my ($self) = @_;
+
+    my $pkg = $self->package;
+    my $type = $self->type;
+    my $processable = $self->processable;
+    my $group = $self->group;
 
     my ($madir, %directories, $built_with_golang, $built_with_octave, %SONAME);
     my ($arch_hardening, $gnu_triplet_re, $ruby_triplet_re);
@@ -130,31 +150,35 @@ sub always {
     my $has_php_ext = 0;
     my $uses_numpy_c_abi = 0;
 
-    my $arch = $info->field('architecture', '');
-    my $multiarch = $info->field('multi-arch', 'no');
-    my $srcpkg = $proc->pkg_src;
+    my $arch = $processable->field('architecture', '');
+    my $multiarch = $processable->field('multi-arch', 'no');
+    my $srcpkg = $processable->source;
 
     $arch_hardening = $HARDENING->value($arch)
       if $arch ne 'all';
 
-    my $src = $group->get_source_processable;
+    my $src = $group->source;
     if (defined($src)) {
-        $built_with_golang = $src->info->relation('build-depends-all')
+        $built_with_golang = $src->relation('build-depends-all')
           ->implies('golang-go | golang-any');
         $built_with_octave
-          = $src->info->relation('build-depends')->implies('dh-octave');
+          = $src->relation('build-depends')->implies('dh-octave');
     } else {
         $built_with_golang = $pkg =~ m/^golang-/;
         $built_with_octave = $pkg =~ m/^octave-/;
     }
 
-    foreach my $file (sort keys %{$info->objdump_info}) {
-        my $objdump = $info->objdump_info->{$file};
+    foreach my $name (sort keys %{$processable->objdump_info}) {
+        my $objdump = $processable->objdump_info->{$name};
         my ($has_lfs, %unharded_functions, @hardened_functions);
         my $is_profiled = 0;
-        # $file can be an object inside a static lib.  These do
+
+        # $name can be an object inside a static lib.  These do
         # not appear in the output of our file_info collection.
-        my $file_info = $info->file_info($file) // '';
+        my $file = $processable->installed->lookup($name);
+        my $file_info = EMPTY;
+        $file_info = $file->file_info
+          if defined $file;
 
         # The LFS check only works reliably for ELF files due to the
         # architecture regex.
@@ -164,13 +188,13 @@ sub always {
               unless defined $ARCH_32_REGEX;
             $has_lfs = 1 unless $file_info =~ m/$ARCH_32_REGEX/o;
             # We don't care if it is a debug file
-            $has_lfs = 1 if $file =~ m,^usr/lib/debug/,;
+            $has_lfs = 1 if $name =~ m,^usr/lib/debug/,;
         }
 
         if (defined $objdump->{SONAME}) {
             foreach my $soname (@{$objdump->{SONAME}}) {
                 $SONAME{$soname} ||= [];
-                push @{$SONAME{$soname}}, $file;
+                push @{$SONAME{$soname}}, $name;
             }
         }
         foreach my $symbol (@{$objdump->{SYMBOLS}}) {
@@ -202,7 +226,7 @@ sub always {
             if ($foo eq 'UND' and $OBSOLETE_CRYPT_FUNCTIONS->known($sym)) {
                 # Using an obsolete DES encryption function.
                 my $tag = $OBSOLETE_CRYPT_FUNCTIONS->value($sym);
-                tag $tag, $file;
+                $self->tag($tag, $name);
             }
 
             next if $is_profiled;
@@ -221,23 +245,23 @@ sub always {
                     $is_profiled = 1;
                 }
             }
-            tag 'binary-compiled-with-profiling-enabled', $file
+            $self->tag('binary-compiled-with-profiling-enabled', $name)
               if $is_profiled;
         }
         if (    %unharded_functions
             and not @hardened_functions
             and not $built_with_golang
             and $arch_hardening->{'hardening-no-fortify-functions'}) {
-            tag 'hardening-no-fortify-functions', $file;
+            $self->tag('hardening-no-fortify-functions', $name);
         }
 
-        tag 'apparently-corrupted-elf-binary', $file
+        $self->tag('apparently-corrupted-elf-binary', $name)
           if $objdump->{'ERRORS'};
-        tag 'binary-file-built-without-LFS-support', $file
+        $self->tag('binary-file-built-without-LFS-support', $name)
           if defined $has_lfs and not $has_lfs;
         if ($objdump->{'BAD-DYNAMIC-TABLE'}) {
-            tag 'binary-with-bad-dynamic-table', $file
-              unless $file =~ m%^usr/lib/debug/%;
+            $self->tag('binary-with-bad-dynamic-table', $name)
+              unless $name =~ m%^usr/lib/debug/%;
         }
     }
 
@@ -300,14 +324,15 @@ sub always {
         }
     }
 
-    tag 'package-name-doesnt-match-sonames', "@sonames"
+    $self->tag('package-name-doesnt-match-sonames', "@sonames")
       if @sonames && !$match_found && $type ne 'udeb';
 
     # process all files in package
-    foreach my $file ($info->sorted_index) {
+    foreach my $file ($processable->installed->sorted_list) {
         my ($fileinfo, $objdump, $fname);
 
-        next if not $file->is_file;
+        next
+          unless $file->is_file;
 
         $fileinfo = $file->file_info;
 
@@ -318,23 +343,26 @@ sub always {
 
         # Warn about Architecture: all packages that contain shared libraries.
         if ($arch eq 'all') {
-            tag 'arch-independent-package-contains-binary-or-object',$file;
+            $self->tag('arch-independent-package-contains-binary-or-object',
+                $file);
         }
 
         $fname = $file->name;
         if ($fname =~ m,^etc/,) {
-            tag 'binary-in-etc', $file;
+            $self->tag('binary-in-etc', $file);
         }
 
         if ($fname =~ m,^usr/share/,) {
-            tag 'arch-dependent-file-in-usr-share', $file;
+            $self->tag('arch-dependent-file-in-usr-share', $file);
         }
 
         if ($multiarch eq 'same') {
             unless ($fname
                 =~ m,\b$gnu_triplet_re(?:\b|_)|/(?:$ruby_triplet_re|java-\d+-openjdk-\Q$arch\E|\.build-id)/,
             ) {
-                tag 'arch-dependent-file-not-in-arch-specific-directory',$file;
+                $self->tag(
+                    'arch-dependent-file-not-in-arch-specific-directory',
+                    $file);
             }
         }
         if ($fileinfo =~ m/\bcurrent ar archive\b/) {
@@ -343,19 +371,18 @@ sub always {
             # unneeded sections in those.
             next if $file =~ m/_g\.a$/;
 
-            $objdump = $info->objdump_info->{$file};
+            $objdump = $processable->objdump_info->{$file};
 
             foreach my $obj (@{ $objdump->{'objects'} }) {
-                my $libobj = $info->objdump_info->{"${file}(${obj})"};
+                my $libobj = $processable->objdump_info->{"${file}(${obj})"};
                 # Shouldn't happen, but...
-                internal_error(
-                    "object ($file $obj) in static lib is missing!?")
+                die "object ($file $obj) in static lib is missing!?"
                   unless defined $libobj;
 
                 if (any { exists($libobj->{'SH'}{$_}) } DEBUG_SECTIONS) {
-                    tag 'unstripped-static-library', "${file}(${obj})";
+                    $self->tag('unstripped-static-library', "${file}(${obj})");
                 } else {
-                    tag_unneeded_sections(
+                    $self->tag_unneeded_sections(
                         'static-library-has-unneeded-section',
                         "${file}(${obj})", $libobj);
                 }
@@ -365,12 +392,12 @@ sub always {
         # ELF?
         next unless $fileinfo =~ m/^[^,]*\bELF\b/o;
 
-        tag 'development-package-ships-elf-binary-in-path', $file
+        $self->tag('development-package-ships-elf-binary-in-path', $file)
           if exists($PATH_DIRECTORIES{$file->dirname})
-          and $info->field('section', 'NONE') =~ m/(?:^|\/)libdevel$/
-          and $info->field('multi-arch', 'NONE') ne 'foreign';
+          and $processable->field('section', 'NONE') =~ m/(?:^|\/)libdevel$/
+          and $processable->field('multi-arch', 'NONE') ne 'foreign';
 
-        $objdump = $info->objdump_info->{$fname};
+        $objdump = $processable->objdump_info->{$fname};
 
         if ($arch eq 'all' or not $ARCH_REGEX->known($arch)) {
             # arch:all or unknown architecture - not much we can say here
@@ -403,20 +430,16 @@ sub always {
                 # Ignore i386 binaries in amd64 packages for right now.
                 $bad = 0 if $fileinfo =~ m/$arch32re/;
             }
-            tag 'binary-from-other-architecture', $file if $bad;
+            $self->tag('binary-from-other-architecture', $file) if $bad;
         }
 
-        my $stringsfd = $info->strings($file);
-        my $strings = do { local $/; <$stringsfd> };
-        close($stringsfd)
-          or warn "Error closing strings fd: $!";
         my $exceptions = {
             %{ $group->info->spelling_exceptions },
             map { $_ => 1} $BINARY_SPELLING_EXCEPTIONS->all
         };
         my $tag_emitter
-          = spelling_tag_emitter('spelling-error-in-binary', $file);
-        check_spelling($strings, $exceptions, $tag_emitter, 0);
+          = $self->spelling_tag_emitter('spelling-error-in-binary', $file);
+        check_spelling($file->strings, $exceptions, $tag_emitter, 0);
 
         # stripped?
         if ($fileinfo =~ m,\bnot stripped\b,o) {
@@ -430,28 +453,30 @@ sub always {
                 or $fname =~ GUILE_PATH_REGEX
                 or $fname =~ m,\.gox$,o) {
                 if (    $fileinfo =~ m/executable/
-                    and $strings =~ m/^Caml1999X0[0-9][0-9]$/m) {
+                    and $file->strings =~ m/^Caml1999X0[0-9][0-9]$/m) {
                     # Check for OCaml custom executables (#498138)
-                    tag 'ocaml-custom-executable', $file;
+                    $self->tag('ocaml-custom-executable', $file);
                 } else {
-                    tag 'unstripped-binary-or-object', $file;
+                    $self->tag('unstripped-binary-or-object', $file);
                 }
             }
         } else {
             # stripped but a debug or profiling library?
             if (($fname =~ m,/lib/debug/,o) or ($fname =~ m,/lib/profile/,o)){
-                tag 'library-in-debug-or-profile-should-not-be-stripped',$file;
+                $self->tag(
+                    'library-in-debug-or-profile-should-not-be-stripped',
+                    $file);
             } else {
                 # appropriately stripped, but is it stripped enough?
-                tag_unneeded_sections('binary-has-unneeded-section', $file,
-                    $objdump);
+                $self->tag_unneeded_sections('binary-has-unneeded-section',
+                    $file,$objdump);
             }
         }
 
         # rpath is disallowed, except in private directories
         if (exists($objdump->{RPATH}) or exists($objdump->{RUNPATH})) {
             if (not %directories) {
-                for my $file ($info->sorted_index) {
+                for my $file ($processable->installed->sorted_list) {
                     my $name;
                     next unless $file->is_dir || $file->is_symlink;
                     $name = $file->name;
@@ -472,19 +497,19 @@ sub always {
                 next
                   if $directories{$rpath}
                   and $rpath !~ m,^(?:/usr)?/lib(?:/$madir)?/?\z,;
-                tag 'binary-or-shlib-defines-rpath', $file, $rpath;
+                $self->tag('binary-or-shlib-defines-rpath', $file, $rpath);
             }
         }
 
         foreach my $emlib ($EMBEDDED_LIBRARIES->all) {
             my $ldata = $EMBEDDED_LIBRARIES->value($emlib);
             if ($ldata->{'source-regex'}) {
-                next if $proc->pkg_src =~ m/^$ldata->{'source-regex'}$/;
+                next if $processable->source =~ m/^$ldata->{'source-regex'}$/;
             } else {
-                next if $proc->pkg_src eq $ldata->{'source'};
+                next if $processable->source eq $ldata->{'source'};
             }
-            if ($strings =~ $ldata->{'match'}) {
-                tag 'embedded-library', "$fname: $ldata->{'libname'}";
+            if ($file->strings =~ $ldata->{'match'}) {
+                $self->tag('embedded-library', "$fname: $ldata->{'libname'}");
             }
         }
 
@@ -492,7 +517,9 @@ sub always {
         next
           unless ($fileinfo =~ m/executable/)
           or ($fileinfo =~ m/shared object/);
-        next if $type eq 'udeb';
+
+        next
+          if $type eq 'udeb';
 
         # Perl library?
         if ($fname =~ m,^usr/lib/(?:[^/]+/)?perl5/.*\.so$,) {
@@ -510,7 +537,8 @@ sub always {
             or(     $fname =~ m,usr/lib/python3/.+\.cpython-\d+([a-z]+)\.so$,
                 and $1 !~ /d/)
         ) {
-            if (index($strings, 'numpy') > -1 and $strings =~ NUMPY_REGEX) {
+            if (index($file->strings, 'numpy') > -1
+                and $file->strings =~ NUMPY_REGEX) {
                 $uses_numpy_c_abi = 1;
             }
         }
@@ -521,9 +549,9 @@ sub always {
             =~ m,^usr/lib/debug/(?:lib\d*|s?bin|usr|opt|dev|emul|\.build-id)/,)
         {
             if (exists($objdump->{NEEDED})) {
-                tag 'debug-file-should-use-detached-symbols', $file;
+                $self->tag('debug-file-should-use-detached-symbols', $file);
             }
-            tag 'debug-file-with-no-debug-symbols', $file
+            $self->tag('debug-file-with-no-debug-symbols', $file)
               unless (exists $objdump->{'SH'}{'.debug_line'}
                 or exists $objdump->{'SH'}{'.zdebug_line'}
                 or exists $objdump->{'SH'}{'.debug_str'}
@@ -534,7 +562,7 @@ sub always {
         if ($fname =~ m,^usr/lib/debug/[^/]+$,) {
             unless (exists($objdump->{NEEDED})
                 || $fileinfo =~ m/statically linked/) {
-                tag 'debug-symbols-directly-in-usr-lib-debug', $file;
+                $self->tag('debug-symbols-directly-in-usr-lib-debug', $file);
             }
         }
 
@@ -558,7 +586,7 @@ sub always {
                                    ld-[\d.]+\.so$
                                 }xsm
                   );
-                tag 'shared-lib-without-dependency-information', $file;
+                $self->tag('shared-lib-without-dependency-information', $file);
             } else {
                 # Some exceptions: files in /boot, /usr/lib/debug/*,
                 # named *-static or *.static, or *-static as
@@ -577,7 +605,7 @@ sub always {
                 next if ($fname =~ m%^usr/lib/debug/%);
                 # ldconfig must be static.
                 next if ($fname eq 'sbin/ldconfig');
-                tag 'statically-linked-binary', $file;
+                $self->tag('statically-linked-binary', $file);
             }
         } else {
             my $no_libc = 1;
@@ -608,40 +636,40 @@ sub always {
                 # but these tend to link against libstdc++ instead.  (see
                 # #719806)
                 if ($is_shared) {
-                    tag 'library-not-linked-against-libc', $file
+                    $self->tag('library-not-linked-against-libc', $file)
                       unless $needs_libcxx ne '';
                 } else {
-                    tag 'program-not-linked-against-libc', $file;
+                    $self->tag('program-not-linked-against-libc', $file);
                 }
             }
 
             if (    $arch_hardening->{'hardening-no-relro'}
                 and not $built_with_golang
                 and not $objdump->{'PH'}{'RELRO'}) {
-                tag 'hardening-no-relro', $file;
+                $self->tag('hardening-no-relro', $file);
             }
 
             if (    $arch_hardening->{'hardening-no-bindnow'}
                 and not $built_with_golang
                 and not exists($objdump->{'FLAGS_1'}{'NOW'})) {
-                tag 'hardening-no-bindnow', $file;
+                $self->tag('hardening-no-bindnow', $file);
             }
 
             if (    $arch_hardening->{'hardening-no-pie'}
                 and not $built_with_golang
                 and $objdump->{'ELF-TYPE'} eq 'EXEC') {
-                tag 'hardening-no-pie', $file;
+                $self->tag('hardening-no-pie', $file);
             }
         }
     }
 
     # Find the package dependencies, which is used by various checks.
-    my $depends = $info->relation('strong');
+    my $depends = $processable->relation('strong');
 
     # Check for a libc dependency.
     if ($needs_depends_line) {
         if ($depends->empty) {
-            tag 'missing-depends-line';
+            $self->tag('missing-depends-line');
         } else {
             if ($needs_libc && $pkg !~ /^libc[\d.]+(?:-|\z)/) {
                 # Match libcXX or libcXX-*, but not libc3p0.
@@ -652,8 +680,8 @@ sub always {
                     if ($needs_libc_count > 0) {
                         $others = " and $needs_libc_count others";
                     }
-                    tag 'missing-dependency-on-libc',
-                      "needed by $needs_libc_file$others";
+                    $self->tag('missing-dependency-on-libc',
+                        "needed by $needs_libc_file$others");
                 }
             }
             if ($needs_libcxx ne '') {
@@ -665,8 +693,10 @@ sub always {
                     if ($needs_libcxx_count > 0) {
                         $others = " and $needs_libcxx_count others";
                     }
-                    tag 'missing-dependency-on-libstdc++',
-                      "needed by $needs_libcxx_file$others";
+                    $self->tag(
+                        'missing-dependency-on-libstdc++',
+                        "needed by $needs_libcxx_file$others"
+                    );
                 }
             }
         }
@@ -678,7 +708,7 @@ sub always {
         # alternatives probably does not make sense here either.
         my $re = qr/^perlapi-[-\w.]+(?:\s*\[[^\]]+\])?$/;
         unless ($depends->matches($re, VISIT_OR_CLAUSE_FULL)) {
-            tag 'missing-dependency-on-perlapi';
+            $self->tag('missing-dependency-on-perlapi');
         }
     }
 
@@ -688,7 +718,7 @@ sub always {
         # alternatives probably does not make sense here either.
         unless ($depends->matches(qr/^phpapi-[\d\w+]+$/, VISIT_OR_CLAUSE_FULL))
         {
-            tag 'missing-dependency-on-phpapi';
+            $self->tag('missing-dependency-on-phpapi');
         }
     }
 
@@ -699,7 +729,7 @@ sub always {
         # defeat the purpose of this relation.  Also, we do not allow
         # versions for -abi as it is a virtual package.
         my $vflags = VISIT_OR_CLAUSE_FULL;
-        tag 'missing-dependency-on-numpy-abi'
+        $self->tag('missing-dependency-on-numpy-abi')
           unless $depends->matches(qr/^python3?-numpy-abi\d+$/, $vflags)
           or (  $depends->matches(qr/^python-numpy \(>[>=][^\|]+$/, $vflags)
             and $depends->matches(qr/^python-numpy \(<[<=][^\|]+$/, $vflags));
@@ -709,10 +739,10 @@ sub always {
 }
 
 sub tag_unneeded_sections {
-    my ($tag, $file, $objdump) = @_;
+    my ($self, $tag, $file, $objdump) = @_;
     foreach my $sect ('.note', '.comment') {
         if (exists $objdump->{'SH'}{$sect}) {
-            tag $tag, "$file $sect";
+            $self->tag($tag, "$file $sect");
         }
     }
     return;
