@@ -19,23 +19,32 @@
 # MA 02110-1301, USA.
 
 package Lintian::nmu;
-use strict;
+
+use v5.20;
 use warnings;
+use utf8;
 use autodie;
 
 use List::MoreUtils qw(any);
 use List::Util qw(first);
 
-use Lintian::Tags qw(tag);
 use Lintian::Util qw(strip);
 
+use Moo;
+use namespace::clean;
+
+with 'Lintian::Check';
+
 sub source {
-    my (undef, undef, $info) = @_;
+    my ($self) = @_;
+
+    my $processable = $self->processable;
+
     my $changelog_mentions_nmu = 0;
     my $changelog_mentions_local = 0;
     my $changelog_mentions_qa = 0;
     my $changelog_mentions_team_upload = 0;
-    my $debian_dir = $info->index_resolved_path('debian/');
+    my $debian_dir = $processable->patched->resolve_path('debian/');
     my $chf;
     $chf = $debian_dir->child('changelog') if $debian_dir;
 
@@ -43,12 +52,12 @@ sub source {
     # looks at debian/changelog in source packages.  Catch a
     # debian/changelog file that's a symlink.
     if ($chf and $chf->is_symlink) {
-        tag 'changelog-is-symlink';
+        $self->tag('changelog-is-symlink');
     }
-    return unless $info->changelog;
+    return unless $processable->changelog;
 
     # Get some data from the changelog file.
-    my ($entry) = @{$info->changelog->entries};
+    my ($entry) = @{$processable->changelog->entries};
     my $uploader = canonicalize($entry->Maintainer // '');
     my $changes = $entry->Changes;
     $changes =~ s/^(\s*\n)+//;
@@ -73,16 +82,16 @@ sub source {
 
     # If the version field is missing, assume it to be a native,
     # maintainer upload as it is probably the most likely case.
-    my $version = $info->field('version', '0-1');
-    my $maintainer = canonicalize($info->field('maintainer', ''));
-    my $uploaders = $info->field('uploaders');
+    my $version = $processable->field('version', '0-1');
+    my $maintainer = canonicalize($processable->field('maintainer', ''));
+    my $uploaders = $processable->field('uploaders');
 
     my $version_nmuness = 0;
     my $version_local = 0;
     my $upload_is_backport = $version =~ m/~bpo(\d+)\+(\d+)$/;
 
     if ($uploader =~ m/^\s|\s$/) {
-        tag 'extra-whitespace-around-name-in-changelog-trailer';
+        $self->tag('extra-whitespace-around-name-in-changelog-trailer');
         strip($uploader);
     }
 
@@ -110,29 +119,28 @@ sub source {
     $upload_is_nmu = 0 if not $uploader;
 
     if ($maintainer =~ /packages\@qa.debian.org/) {
-        tag 'orphaned-package-should-not-have-uploaders'
+        $self->tag('orphaned-package-should-not-have-uploaders')
           if defined $uploaders;
-        tag 'qa-upload-has-incorrect-version-number', $version
+        $self->tag('qa-upload-has-incorrect-version-number', $version)
           if $version_nmuness == 1;
-        tag 'changelog-should-mention-qa'
-          if !$changelog_mentions_qa;
+        $self->tag('changelog-should-mention-qa') if !$changelog_mentions_qa;
     } elsif ($changelog_mentions_team_upload) {
-        tag 'team-upload-has-incorrect-version-number', $version
+        $self->tag('team-upload-has-incorrect-version-number', $version)
           if $version_nmuness == 1;
-        tag 'unnecessary-team-upload' unless $upload_is_nmu;
+        $self->tag('unnecessary-team-upload') unless $upload_is_nmu;
     } else {
         # Local packages may be either NMUs or not.
         unless ($changelog_mentions_local || $version_local) {
-            tag 'changelog-should-mention-nmu'
+            $self->tag('changelog-should-mention-nmu')
               if !$changelog_mentions_nmu && $upload_is_nmu;
-            tag 'source-nmu-has-incorrect-version-number', $version
+            $self->tag('source-nmu-has-incorrect-version-number', $version)
               if $upload_is_nmu
               && $version_nmuness != 1
               && !$upload_is_backport;
         }
-        tag 'changelog-should-not-mention-nmu'
+        $self->tag('changelog-should-not-mention-nmu')
           if $changelog_mentions_nmu && !$upload_is_nmu;
-        tag 'maintainer-upload-has-incorrect-version-number', $version
+        $self->tag('maintainer-upload-has-incorrect-version-number', $version)
           if !$upload_is_nmu && $version_nmuness;
     }
 
