@@ -23,8 +23,10 @@
 # MA 02110-1301, USA.
 
 package Lintian::scripts;
-use strict;
+
+use v5.20;
 use warnings;
+use utf8;
 use autodie;
 
 use Capture::Tiny qw(capture);
@@ -32,11 +34,15 @@ use List::MoreUtils qw(any);
 use POSIX qw(strftime);
 use Try::Tiny;
 
-use Lintian::Check qw($known_shells_regex);
+use Lintian::Spelling qw($known_shells_regex);
 use Lintian::Data;
 use Lintian::Relation;
-use Lintian::Tags qw(tag);
-use Lintian::Util qw(internal_error safe_qx strip);
+use Lintian::Util qw(safe_qx strip);
+
+use Moo;
+use namespace::clean;
+
+with 'Lintian::Check';
 
 # This is a map of all known interpreters.  The key is the interpreter
 # name (the binary invoked on the #! line).  The value is an anonymous
@@ -102,8 +108,7 @@ my $BAD_MAINT_CMD = Lintian::Data->new(
     sub {
         my @sliptline = split(/\s*\~\~/, $_[1], 5);
         if(scalar(@sliptline) != 5) {
-            internal_error(
-                'Syntax error in scripts/maintainer-script-bad-command:', $.);
+            die "Syntax error in scripts/maintainer-script-bad-command: $.";
         }
         my ($incat,$inauto,$exceptinpackage,$inscript,$regexp) = @sliptline;
         $regexp =~ s/\$[{]LEADIN[}]/$LEADINSTR/;
@@ -229,42 +234,45 @@ my @bashism_regexs = (
 
 # a local function to help use separate tags for example scripts
 sub script_tag {
-    my($tag, $filename, @rest) = @_;
+    my($self, $tag, $filename, @rest) = @_;
 
     $tag = "example-$tag"
       if $filename and $filename =~ m,usr/share/doc/[^/]+/examples/,;
 
-    tag($tag, $filename, @rest);
+    $self->tag(($tag, $filename, @rest));
     return;
 }
 
-sub binary {
-    my ($pkg, undef, $info) = @_;
+sub installable {
+    my ($self) = @_;
+
+    my $pkg = $self->package;
+    my $processable = $self->processable;
 
     my (%executable, %ELF, %scripts, %seen_helper_cmds);
 
     # no dependency for install-menu, because the menu package specifically
     # says not to depend on it.
 
-    foreach my $file ($info->sorted_index) {
+    foreach my $file ($processable->installed->sorted_list) {
         next if not $file->is_file;
         $ELF{$file} = 1 if $file->file_info =~ /^[^,]*\bELF\b/o;
         next unless $file->operm & 0111;
         $executable{$file} = 1;
     }
 
-    my $all_parsed = Lintian::Relation->and($info->relation('all'),
-        $info->relation('provides'),$pkg);
-    my $str_deps = $info->relation('strong');
+    my $all_parsed = Lintian::Relation->and($processable->relation('all'),
+        $processable->relation('provides'),$pkg);
+    my $str_deps = $processable->relation('strong');
 
     my @x11_fonts
       = grep {m,^usr/share/fonts/X11/.*\.(?:afm|pcf|pfa|pfb)(?:\.gz)?$,}
-      $info->sorted_index;
+      $processable->installed->sorted_list;
 
     my %old_versions;
     for my $entry (
-        $info->changelog
-        ? @{$info->changelog->entries}
+        $processable->changelog
+        ? @{$processable->changelog->entries}
         : ()
     ) {
         my $timestamp = $entry->Timestamp // $OLDSTABLE_RELEASE;
@@ -272,9 +280,15 @@ sub binary {
           if $timestamp < $OLDSTABLE_RELEASE;
     }
 
-    for my $filename (sort keys %{$info->scripts}) {
-        my $interpreter = $info->scripts->{$filename}{interpreter};
-        my $calls_env = $info->scripts->{$filename}{calls_env};
+    for my $file ($processable->installed->sorted_list) {
+
+        next
+          unless $file->is_script;
+
+        my $interpreter = $file->script->{interpreter};
+        my $calls_env = $file->script->{calls_env};
+
+        my $filename = $file->name;
         my $path;
         $scripts{$filename} = 1;
 
@@ -290,7 +304,8 @@ sub binary {
 
         # no checks necessary at all for scripts in /usr/share/doc/
         # unless they are examples
-        next if $in_docs and not $in_examples;
+        next
+          if $in_docs && !$in_examples;
 
         my ($base) = $interpreter =~ m,([^/]*)$,;
 
@@ -305,13 +320,13 @@ sub binary {
           if (  $filename =~ m,\.in$,
             and $interpreter =~ m,^(\@|<\<)[A-Z_]+(\@|>\>)$,);
 
-        my $is_absolute = ($interpreter =~ m,^/, or defined $calls_env);
+        my $is_absolute = ($interpreter =~ m,^/, or $calls_env);
 
         # As a special-exception, Policy 10.4 states that Perl scripts must use
         # /usr/bin/perl directly and not via /usr/bin/env, etc.
-        script_tag(bad_interpreter_tag_name('/usr/bin/env perl'),
+        $self->script_tag(bad_interpreter_tag_name('/usr/bin/env perl'),
             $filename, '(#!/usr/bin/env perl != /usr/bin/perl)')
-          if defined $calls_env and $interpreter eq 'perl';
+          if $calls_env and $interpreter eq 'perl';
 
         # Skip files that have the #! line, but are not executable and
         # do not have an absolute path and are not in a bin/ directory
@@ -340,15 +355,21 @@ sub binary {
           if $filename =~ m,^usr/share/cargo/registry/,;
 
         if ($interpreter eq '') {
-            script_tag('script-without-interpreter', $filename);
+            $self->script_tag('script-without-interpreter', $filename);
             next;
         }
 
         # Either they use an absolute path or they use '/usr/bin/env interp'.
-        script_tag('interpreter-not-absolute', $filename, "#!$interpreter")
+        $self->script_tag('interpreter-not-absolute', $filename,
+            "#!$interpreter")
           unless $is_absolute;
-        tag 'script-not-executable', $filename
-          unless ($executable{$filename}
+
+        my $bash_completion_regex
+          = qr{^usr/share/bash-completion/completions/.*};
+
+        $self->tag('script-not-executable', $filename)
+          unless (
+               $executable{$filename}
             or $filename =~ m,^usr/(?:lib|share)/.*\.pm,
             or $filename =~ m,^usr/(?:lib|share)/.*\.py,
             or $filename =~ m,^usr/(?:lib|share)/ruby/.*\.rb,
@@ -358,33 +379,39 @@ sub binary {
             or $filename =~ m,\.ex$,
             or $filename eq 'etc/init.d/skeleton'
             or $filename =~ m,^etc/menu-methods,
-            or $filename =~ m,^etc/X11/Xsession\.d,)
-          or $in_docs;
+            or $filename =~ $bash_completion_regex,
+            or $filename =~ m,^etc/X11/Xsession\.d,
+          )or $in_docs;
+
+        # for bash completion issue this instead
+        $self->tag('bash-completion-with-hashbang', $filename)
+          if $filename =~ $bash_completion_regex;
 
         # Warn about csh scripts.
-        tag 'csh-considered-harmful', $filename
+        $self->tag('csh-considered-harmful', $filename)
           if (  ($base eq 'csh' or $base eq 'tcsh')
             and $executable{$filename}
             and $filename !~ m,^etc/csh/login\.d/,)
           and not $in_docs;
 
-        $path = $info->index_resolved_path($filename);
-        next if not $path or not $path->is_open_ok;
+        next
+          unless $file->is_open_ok;
         # Syntax-check most shell scripts, but don't syntax-check
         # scripts that end in .dpatch.  bash -n doesn't stop checking
         # at exit 0 and goes on to blow up on the patch itself.
         if ($base =~ /^$known_shells_regex$/) {
             if (
                     -x $interpreter
-                and not script_is_evil_and_wrong($path)
+                and not script_is_evil_and_wrong($file)
                 and $filename !~ m,\.dpatch$,
                 and $filename !~ m,\.erb$,
                 # exclude some shells. zsh -n is broken, see #485885
                 and $base !~ m/^(?:z|t?c)sh$/
             ) {
 
-                if (check_script_syntax($interpreter, $path)) {
-                    script_tag('shell-script-fails-syntax-check', $filename);
+                if (check_script_syntax($interpreter, $file)) {
+                    $self->script_tag('shell-script-fails-syntax-check',
+                        $filename);
                 }
             }
         }
@@ -406,20 +433,22 @@ sub binary {
         }
         if ($data) {
             my $expected = $data->[0] . '/' . $base;
-            unless ($interpreter eq $expected or defined $calls_env) {
-                script_tag(bad_interpreter_tag_name($expected),
+            unless ($interpreter eq $expected or $calls_env) {
+                $self->script_tag(bad_interpreter_tag_name($expected),
                     $filename, "(#!$interpreter != $expected)");
             }
         } elsif ($interpreter =~ m,/usr/local/,) {
-            script_tag('interpreter-in-usr-local', $filename,"#!$interpreter");
+            $self->script_tag('interpreter-in-usr-local', $filename,
+                "#!$interpreter");
         } elsif ($interpreter eq '/bin/env') {
-            script_tag('script-uses-bin-env', $filename);
+            $self->script_tag('script-uses-bin-env', $filename);
         } elsif ($interpreter eq 'nodejs') {
-            script_tag('script-uses-deprecated-nodejs-location',$filename);
+            $self->script_tag('script-uses-deprecated-nodejs-location',
+                $filename);
             # Check whether we have correct dependendies on nodejs regardless.
             $data = $INTERPRETERS->value('node');
         } elsif ($base =~ /^php/) {
-            script_tag('php-script-with-unusual-interpreter',
+            $self->script_tag('php-script-with-unusual-interpreter',
                 $filename, "$interpreter");
 
             # This allows us to still perform the dependencies checks
@@ -432,7 +461,7 @@ sub binary {
                 # executable).
                 my $interfile = substr $interpreter, 1;
                 $pinter = 1 if $executable{$interfile};
-            } elsif (defined $calls_env) {
+            } elsif ($calls_env) {
                 for my $dir (qw(usr/bin bin)) {
                     if ($executable{"$dir/$interpreter"}) {
                         $pinter = 1;
@@ -440,7 +469,8 @@ sub binary {
                     }
                 }
             }
-            script_tag('unusual-interpreter', $filename, "#!$interpreter")
+            $self->script_tag('unusual-interpreter', $filename,
+                "#!$interpreter")
               unless $pinter;
 
         }
@@ -451,7 +481,7 @@ sub binary {
             &&!$str_deps->implies(
                 'libperl4-corelibs-perl | perl (<< 5.12.3-7)')
         ) {
-            my $fd = $path->open;
+            open(my $fd, '<', $file->unpacked_path);
             while (<$fd>) {
                 if (
                     m{ (?:do|require)\s+['"] # do/require
@@ -467,8 +497,8 @@ sub binary {
                           \.pl['"]
                }xsm
                 ) {
-                    tag 'script-uses-perl4-libs-without-dep',
-                      "$filename:$. ${1}.pl";
+                    $self->tag('script-uses-perl4-libs-without-dep',
+                        "$filename:$. ${1}.pl");
                 }
             }
             close($fd);
@@ -486,11 +516,13 @@ sub binary {
             }
             if ($depends && !$all_parsed->implies($depends)) {
                 if ($base =~ /^php/) {
-                    tag 'php-script-but-no-php-cli-dep', $filename,
-                      "#!$interpreter";
+                    $self->tag('php-script-but-no-php-cli-dep',
+                        $filename,"#!$interpreter");
                 } elsif ($base =~ /^(python|ruby|[mg]awk)$/) {
-                    tag("$base-script-but-no-$base-dep",
-                        $filename, "#!$interpreter");
+                    $self->tag((
+                        "$base-script-but-no-$base-dep",$filename,
+                        "#!$interpreter"
+                    ));
                 } elsif ($base eq 'csh' && $filename =~ m,^etc/csh/login\.d/,){
                     # Initialization files for csh.
                 } elsif ($base eq 'fish' && $filename =~ m,^etc/fish\.d/,) {
@@ -505,8 +537,10 @@ sub binary {
                     && $all_parsed->matches(qr/^erlang-abi-[\d+\.]+$/)) {
                     # ABI-versioned virtual packages for erlang
                 } else {
-                    tag 'missing-dep-for-interpreter', "$base => $depends",
-                      "($filename)", "#!$interpreter";
+                    $self->tag(
+                        'missing-dep-for-interpreter', "$base => $depends",
+                        "($filename)", "#!$interpreter"
+                    );
                 }
             }
         } elsif ($VERSIONED_INTERPRETERS->known($base)) {
@@ -520,10 +554,13 @@ sub binary {
             my $depends = join(' | ',  @depends);
             unless ($all_parsed->implies($depends)) {
                 if ($base =~ /^(wish|tclsh)/) {
-                    tag "$1-script-but-no-$1-dep", $filename, "#!$interpreter";
+                    $self->tag("$1-script-but-no-$1-dep", $filename,
+                        "#!$interpreter");
                 } else {
-                    tag 'missing-dep-for-interpreter', "$base => $depends",
-                      "($filename)", "#!$interpreter";
+                    $self->tag(
+                        'missing-dep-for-interpreter', "$base => $depends",
+                        "($filename)", "#!$interpreter"
+                    );
                 }
             }
         } else {
@@ -532,96 +569,103 @@ sub binary {
             $depends =~ s/\$1/$version/g;
             unless ($all_parsed->implies($depends)) {
                 if ($base =~ /^(python|ruby)/) {
-                    tag "$1-script-but-no-$1-dep", $filename,"#!$interpreter";
+                    $self->tag("$1-script-but-no-$1-dep", $filename,
+                        "#!$interpreter");
                 } else {
-                    tag 'missing-dep-for-interpreter', "$base => $depends",
-                      "($filename)", "#!$interpreter";
+                    $self->tag(
+                        'missing-dep-for-interpreter', "$base => $depends",
+                        "($filename)", "#!$interpreter"
+                    );
                 }
             }
         }
     }
 
-    foreach (keys %executable) {
-        my $index_info = $info->index($_);
+    for my $name (keys %executable) {
+        my $file = $processable->installed->lookup($name);
         my $ok = 0;
-        if ($index_info->is_hardlink) {
+        if ($file->is_hardlink) {
             # We don't collect script information for hardlinks, so check
             # if the target is a script.
-            my $target = $index_info->link_normalized;
-            if (exists $info->scripts->{$target}) {
-                $ok = 1;
-            }
+            my $targetname = $file->link_normalized;
+            my $target = $processable->installed->lookup($targetname);
+
+            $ok = 1
+              if $target->is_script;
         }
 
-        tag 'executable-not-elf-or-script', $_
+        $self->tag('executable-not-elf-or-script', $name)
           unless (
                $ok
-            or $ELF{$_}
-            or $scripts{$_}
-            or $_ =~ m,^usr(?:/X11R6)?/man/,
-            or $_ =~ m/\.exe$/ # mono convention
-            or $_ =~ m/\.jar$/ # Debian Java policy 2.2
+            or $ELF{$name}
+            or $scripts{$name}
+            or $name =~ m,^usr(?:/X11R6)?/man/,
+            or $name =~ m/\.exe$/ # mono convention
+            or $name =~ m/\.jar$/ # Debian Java policy 2.2
           );
     }
 
-    open(my $ctrl_fd, '<', $info->lab_data_path('control-scripts'));
+    # get maintainer scripts
+    my @control
+      = grep { $_->is_control } $self->processable->control->sorted_list;
 
     # Handle control scripts.  This is an edited version of the code for
     # normal scripts above, because there were just enough differences to
     # make a shared function awkward.
 
-    my (%added_diversions, %removed_diversions, %dh_cmd_substs);
+    my (%added_diversions, %removed_diversions);
     my $expand_diversions = 0;
-    while (<$ctrl_fd>) {
-        chop;
+    for my $file (@control) {
 
-        m/^(\S*) (.*)$/
-          or internal_error("bad line in control-scripts file: $_");
-        my $interpreter = $1;
-        my $file = $2;
-        my $path = $info->control_index_resolved_path($file);
+        my $interpreter = $file->control->{interpreter};
 
         $interpreter =~ m|([^/]*)$|;
         my $base = $1;
 
         # tag for statistics
-        tag 'maintainer-script-interpreter', "control/$file", $interpreter;
+        $self->tag('maintainer-script-interpreter',
+            "control/$file", $interpreter);
 
         if ($interpreter eq '') {
-            tag 'script-without-interpreter', "control/$file";
+            $self->tag('script-without-interpreter', "control/$file");
             next;
         }
 
         if ($interpreter eq 'ELF') {
-            tag 'elf-maintainer-script', "control/$file";
+            $self->tag('elf-maintainer-script', "control/$file");
             next;
         }
 
-        tag 'interpreter-not-absolute', "control/$file", "#!$interpreter"
+        $self->tag('interpreter-not-absolute', "control/$file",
+            "#!$interpreter")
           unless ($interpreter =~ m|^/|);
 
         if ($interpreter =~ m|/usr/local/|) {
-            tag 'control-interpreter-in-usr-local', "control/$file",
-              "#!$interpreter";
+            $self->tag('control-interpreter-in-usr-local',
+                "control/$file","#!$interpreter");
         } elsif ($base eq 'sh' or $base eq 'bash' or $base eq 'perl') {
             my $expected = ($INTERPRETERS->value($base))->[0] . '/' . $base;
-            tag bad_interpreter_tag_name($expected),
-              "#!$interpreter != $expected","(control/$file)"
-              unless $interpreter eq $expected;
+            $self->tag(
+                bad_interpreter_tag_name($expected),
+                "#!$interpreter != $expected",
+                "(control/$file)"
+            ) unless $interpreter eq $expected;
         } elsif ($file eq 'config') {
-            tag 'forbidden-config-interpreter', "#!$interpreter";
+            $self->tag('forbidden-config-interpreter', "#!$interpreter");
         } elsif ($file eq 'postrm') {
-            tag 'forbidden-postrm-interpreter', "#!$interpreter";
+            $self->tag('forbidden-postrm-interpreter', "#!$interpreter");
         } elsif ($INTERPRETERS->known($base)) {
             my $data = $INTERPRETERS->value($base);
             my $expected = $data->[0] . '/' . $base;
             unless ($interpreter eq $expected) {
-                tag bad_interpreter_tag_name($expected),
-                  "#!$interpreter != $expected",
-                  "(control/$file)";
+                $self->tag(
+                    bad_interpreter_tag_name($expected),
+                    "#!$interpreter != $expected",
+                    "(control/$file)"
+                );
             }
-            tag 'unusual-control-interpreter', "control/$file",
-              "#!$interpreter";
+            $self->tag('unusual-control-interpreter', "control/$file",
+                "#!$interpreter");
 
             # Interpreters used by preinst scripts must be in
             # Pre-Depends.  Interpreters used by postinst or prerm
@@ -629,30 +673,35 @@ sub binary {
             unless (not $data->[1]) {
                 my $depends = Lintian::Relation->new($data->[1]);
                 if ($file eq 'preinst') {
-                    unless ($info->relation('pre-depends')->implies($depends)){
-                        tag 'preinst-interpreter-without-predepends',
-                          "#!$interpreter";
+                    unless ($processable->relation('pre-depends')
+                        ->implies($depends)){
+                        $self->tag('preinst-interpreter-without-predepends',
+                            "#!$interpreter");
                     }
                 } else {
-                    unless ($info->relation('strong')->implies($depends)) {
-                        tag 'control-interpreter-without-depends',
-                          "control/$file",
-                          "#!$interpreter";
+                    unless (
+                        $processable->relation('strong')->implies($depends)) {
+                        $self->tag(
+                            'control-interpreter-without-depends',
+                            "control/$file",
+                            "#!$interpreter"
+                        );
                     }
                 }
             }
         } else {
-            tag 'unknown-control-interpreter', "control/$file",
-              "#!$interpreter";
+            $self->tag('unknown-control-interpreter', "control/$file",
+                "#!$interpreter");
             next; # no use doing further checks if it's not a known interpreter
         }
 
         # perhaps we should warn about *csh even if they're somehow screwed,
         # but that's not really important...
-        tag 'csh-considered-harmful', "control/$file"
+        $self->tag('csh-considered-harmful', "control/$file")
           if ($base eq 'csh' or $base eq 'tcsh');
 
-        next if not $path or not $path->is_open_ok;
+        next
+          unless $file->is_open_ok;
 
         my $shellscript = $base =~ /^$known_shells_regex$/ ? 1 : 0;
 
@@ -661,14 +710,15 @@ sub binary {
         if ($shellscript) {
             $checkbashisms = $base eq 'sh' ? 1 : 0;
             if ($base eq 'sh' or $base eq 'bash') {
-                if (check_script_syntax("/bin/${base}", $path)) {
-                    tag 'maintainer-shell-script-fails-syntax-check', $file;
+                if (check_script_syntax("/bin/${base}", $file)) {
+                    $self->tag('maintainer-shell-script-fails-syntax-check',
+                        $file);
                 }
             }
         }
 
         # now scan the file contents themselves
-        my $fd = $path->open;
+        open(my $fd, '<', $file->unpacked_path);
 
         my (
             $saw_init, $saw_invoke,
@@ -688,21 +738,23 @@ sub binary {
             }
 
             if (/\#DEBHELPER\#/) {
-                tag 'maintainer-script-has-unexpanded-debhelper-token', $file;
-            }
-            if (/^# Automatically added by (\S+)\s*$/) {
-                my $dh_cmd = $1;
-                # dh_python puts a trailing ":", remove that.
-                $dh_cmd =~ s/:++$//g;
-                tag 'debhelper-autoscript-in-maintainer-scripts', $dh_cmd
-                  if not $dh_cmd_substs{$dh_cmd}++;
-                $in_automatic_section = 1;
+                $self->tag('maintainer-script-has-unexpanded-debhelper-token',
+                    $file);
             }
+
+            $in_automatic_section = 1
+              if /^# Automatically added by \S+\s*$/;
+
             $in_automatic_section = 0
               if $_ eq '# End automatically added section';
 
-            next if m,^\s*$,;  # skip empty lines
-            next if m,^\s*\#,; # skip comment lines
+            # skip empty lines
+            next
+              if m,^\s*$,;
+
+            # skip comment lines
+            next
+              if m,^\s*\#,;
             $_ = remove_comments($_);
 
             # Concatenate lines containing continuation character (\)
@@ -754,7 +806,7 @@ sub binary {
 
             $saw_udevadm_guard = 1 if m/\b(if|which|command)\s+.*udevadm/g;
             if (m,$LEADIN(?:/bin/)?udevadm\s, and $saw_sete) {
-                tag 'udevadm-called-without-guard', "$file:$."
+                $self->tag('udevadm-called-without-guard', "$file:$.")
                   unless $saw_udevadm_guard
                   or m/\|\|/
                   or $str_deps->implies('udev');
@@ -766,19 +818,19 @@ sub binary {
                 and not m/\bmkdir\b/
                 and not m/\bXXXXXX\b/
                 and not m/\$RANDOM/) {
-                #<<< no perltidy - tag name too long
-                tag 'possibly-insecure-handling-of-tmp-files-in-maintainer-script',
-                #>>>
-                  "$file:$."
-                  unless $warned{tmp};
+                $self->tag(
+'possibly-insecure-handling-of-tmp-files-in-maintainer-script',
+                    "$file:$."
+                ) unless $warned{tmp};
                 $warned{tmp} = 1;
             }
             if (m/^\s*killall(?:\s|\z)/) {
-                tag 'killall-is-dangerous', "$file:$." unless $warned{killall};
+                $self->tag('killall-is-dangerous', "$file:$.")
+                  unless $warned{killall};
                 $warned{killall} = 1;
             }
             if (m/^\s*mknod(?:\s|\z)/ and not m/\sp\s/) {
-                tag 'mknod-in-maintainer-script', "$file:$.";
+                $self->tag('mknod-in-maintainer-script', "$file:$.");
             }
 
             # Collect information about init script invocations to
@@ -908,8 +960,8 @@ sub binary {
                     }
 
                     if ($found) {
-                        tag 'possible-bashism-in-maintainer-script',
-                          "$file:$. \'$match\'";
+                        $self->tag('possible-bashism-in-maintainer-script',
+                            "$file:$. \'$match\'");
                     }
 
                     # Only look for the beginning of a heredoc here,
@@ -923,28 +975,31 @@ sub binary {
                     }
                 }
                 if (!$cat_string) {
-                    generic_check_bad_command($_, $file, $., $pkg, 0,
+                    $self->generic_check_bad_command($_, $file, $., 0,
                         $in_automatic_section);
 
                     if (m,/usr/share/debconf/confmodule,) {
                         $saw_debconf = 1;
                     }
                     if (m/^\s*read(?:\s|\z)/ && !$saw_debconf) {
-                        tag 'read-in-maintainer-script', "$file:$.";
+                        $self->tag('read-in-maintainer-script', "$file:$.");
                     }
 
-                    tag 'multi-arch-same-package-calls-pycompile', "$file:$."
+                    $self->tag('multi-arch-same-package-calls-pycompile',
+                        "$file:$.")
                       if m/^\s*py3?compile(?:\s|\z)/
-                      and $info->field('multi-arch', 'no') eq 'same';
+                      and $processable->field('multi-arch', 'no') eq 'same';
 
                     if (m,>\s*/etc/inetd\.conf(?:\s|\Z),) {
-                        tag 'maintainer-script-modifies-inetd-conf',"$file:$."
-                          unless $info->relation('provides')
+                        $self->tag('maintainer-script-modifies-inetd-conf',
+                            "$file:$.")
+                          unless $processable->relation('provides')
                           ->implies('inet-superserver');
                     }
                     if (m,^\s*(?:cp|mv)\s+(?:.*\s)?/etc/inetd\.conf\s*$,) {
-                        tag 'maintainer-script-modifies-inetd-conf',"$file:$."
-                          unless $info->relation('provides')
+                        $self->tag('maintainer-script-modifies-inetd-conf',
+                            "$file:$.")
+                          unless $processable->relation('provides')
                           ->implies('inet-superserver');
                     }
 
@@ -965,8 +1020,9 @@ sub binary {
                                       (/(?:usr/)?s?bin/[\w.+-]+)
                                       (?:\s|;|\Z)}xsm
                         ) {
-                            tag 'command-with-path-in-maintainer-script',
-                              "$file:$. $1"
+                            $self->tag(
+                                'command-with-path-in-maintainer-script',
+                                "$file:$. $1")
                               unless $in_automatic_section;
                         }
                     }
@@ -978,16 +1034,16 @@ sub binary {
                           (/(?:usr/)?s?bin/[\w.+-]+)
                           \s+ \]}xsm
                     ){
-                        tag 'command-with-path-in-maintainer-script',
-                          "$file:$. $1"
+                        $self->tag('command-with-path-in-maintainer-script',
+                            "$file:$. $1")
                           unless $in_automatic_section;
                     }
 
                     $cmd =~ s/\`[^\`]+\`//g;
                     if ($cmd =~ m,$LEADIN(/(?:usr/)?s?bin/[\w.+-]+)(?:\s|;|$),)
                     {
-                        tag 'command-with-path-in-maintainer-script',
-                          "$file:$. $1"
+                        $self->tag('command-with-path-in-maintainer-script',
+                            "$file:$. $1")
                           unless $in_automatic_section;
                     }
                 }
@@ -1003,14 +1059,14 @@ sub binary {
                             or m,command\s+.*?$regex,) {
                             $warned{$package} = 1;
                         } elsif (!/\|\|\s*true\b/) {
-                            unless (
-                                $info->relation('strong')->implies($package)) {
+                            unless ($processable->relation('strong')
+                                ->implies($package)) {
                                 my $shortpackage = $package;
                                 $shortpackage =~ s/[ \(].*//;
-                                #<<< no perltidy - tag name too long
-                                tag "maintainer-script-needs-depends-on-$shortpackage",
-                                #>>>
-                                  $file;
+                                $self->tag(
+"maintainer-script-needs-depends-on-$shortpackage",
+                                    $file
+                                );
                                 $warned{$package} = 1;
                             }
                         }
@@ -1018,46 +1074,53 @@ sub binary {
                 }
             }
 
-            generic_check_bad_command($_, $file, $., $pkg, 1,
+            $self->generic_check_bad_command($_, $file, $., 1,
                 $in_automatic_section);
 
             for my $ver (sort keys %old_versions) {
                 next if $ver =~ /^\d+$/;
-                #<<< no perltidy
-                if (m,$LEADIN(?:/usr/bin/)?dpkg\s+--compare-versions\s+.*\b\Q$ver\E(?!\.)\b,) {
-                    my $date = strftime('%Y-%m-%d', gmtime $old_versions{$ver});
-                    my $epoch = strftime('%Y-%m-%d', gmtime $OLDSTABLE_RELEASE);
-                    tag 'maintainer-script-supports-ancient-package-version',
-                      "$file:$.", $ver, "($date < $epoch)";
+                if (
+m,$LEADIN(?:/usr/bin/)?dpkg\s+--compare-versions\s+.*\b\Q$ver\E(?!\.)\b,
+                ) {
+                    my $date= strftime('%Y-%m-%d', gmtime $old_versions{$ver});
+                    my $epoch= strftime('%Y-%m-%d', gmtime $OLDSTABLE_RELEASE);
+                    $self->tag(
+                        'maintainer-script-supports-ancient-package-version',
+                        "$file:$.", $ver, "($date < $epoch)");
                     last;
                 }
-                #>>>
             }
 
             if (m,$LEADIN(?:/usr/sbin/)?update-inetd\s,) {
-                tag 'maintainer-script-has-invalid-update-inetd-options',
-                  "$file:$.", '(--pattern with --add)'
+                $self->tag(
+                    'maintainer-script-has-invalid-update-inetd-options',
+                    "$file:$.", '(--pattern with --add)')
                   if /--pattern/ && /--add/;
-                tag 'maintainer-script-has-invalid-update-inetd-options',
-                  "$file:$.", '(--group without --add)'
+                $self->tag(
+                    'maintainer-script-has-invalid-update-inetd-options',
+                    "$file:$.", '(--group without --add)')
                   if /--group/ && !/--add/;
             }
 
-            my $pdepends = $info->relation('pre-depends');
-            tag 'skip-systemd-native-flag-missing-pre-depends', "$file:$."
+            my $pdepends = $processable->relation('pre-depends');
+            $self->tag('skip-systemd-native-flag-missing-pre-depends',
+                "$file:$.")
               if m/invoke-rc.d\b.*--skip-systemd-native\b/
               && !$pdepends->implies('init-system-helpers (>= 1.54~)');
 
-            my $depends = $info->relation('depends');
-            tag 'missing-versioned-depends-on-init-system-helpers',
-              "$file:$.", "\"$1 $2\" needs init-system-helpers >= 1.50"
+            my $depends = $processable->relation('depends');
+            $self->tag(
+                'missing-versioned-depends-on-init-system-helpers',
+                "$file:$.",
+                "\"$1 $2\" needs init-system-helpers >= 1.50"
+              )
               if m/(update-rc\.d)\b.*(defaults-disabled)\b/
               && !$depends->implies('init-system-helpers (>= 1.50)');
 
             if (m,$LEADIN(?:/usr/sbin/)?dpkg-divert\s,
                 && !/--(?:help|list|truename|version)/) {
                 if (/--local/) {
-                    tag 'package-uses-local-diversion', "$file:$.";
+                    $self->tag('package-uses-local-diversion', "$file:$.");
                 }
                 my $mode = /--remove/ ? 'remove' : 'add';
                 my ($divert) = /dpkg-divert\s*(.*)$/;
@@ -1109,40 +1172,39 @@ sub binary {
                     push @{$removed_diversions{$divert}},
                       {'script' => $file, 'line' => $.};
                 } else {
-                    internal_error("\$mode has unknown value: $mode");
+                    die "\$mode has unknown value: $mode";
                 }
             }
         }
 
         foreach my $font (@x11_fonts) {
-            tag 'missing-call-to-update-fonts', $font
+            $self->tag('missing-call-to-update-fonts', $font)
               if $file eq 'postinst' and not $saw_update_fonts;
         }
 
         if ($saw_init && !$saw_invoke) {
-            tag 'maintainer-script-calls-init-script-directly',
-              "$file:$saw_init";
+            $self->tag('maintainer-script-calls-init-script-directly',
+                "$file:$saw_init");
         }
         unless ($has_code) {
-            tag 'maintainer-script-empty', $file;
+            $self->tag('maintainer-script-empty', $file);
         }
         if ($shellscript && !$saw_sete) {
             if ($saw_bange) {
-                tag 'maintainer-script-without-set-e', $file;
+                $self->tag('maintainer-script-without-set-e', $file);
             } else {
-                tag 'maintainer-script-ignores-errors', $file;
+                $self->tag('maintainer-script-ignores-errors', $file);
             }
         }
 
         if ($saw_statoverride_add && !$saw_statoverride_list) {
-            tag 'unconditional-use-of-dpkg-statoverride',
-              "$file:$saw_statoverride_add";
+            $self->tag('unconditional-use-of-dpkg-statoverride',
+                "$file:$saw_statoverride_add");
         }
 
         close($fd);
 
     }
-    close($ctrl_fd);
 
     for my $cmd (qw(rm_conffile mv_conffile symlink_to_dir)) {
         next unless $seen_helper_cmds{$cmd};
@@ -1150,7 +1212,8 @@ sub binary {
         # dpkg-maintscript-helper(1) recommends the snippets are in all
         # maintainer scripts but they are not strictly required in prerm.
         for my $file (qw(preinst postinst postrm)) {
-            tag 'missing-call-to-dpkg-maintscript-helper', "$file ($cmd)"
+            $self->tag('missing-call-to-dpkg-maintscript-helper',
+                "$file ($cmd)")
               unless $seen_helper_cmds{$cmd}{$file};
         }
     }
@@ -1222,7 +1285,8 @@ sub binary {
 
                 $divert = unquote($divert, $expand_diversions);
 
-                tag 'remove-of-unknown-diversion', $divert, "$script:$line";
+                $self->tag('remove-of-unknown-diversion', $divert,
+                    "$script:$line");
             }
         }
     }
@@ -1235,7 +1299,7 @@ sub binary {
         $divert = unquote($divert, $expand_diversions);
 
         if (not exists $added_diversions{$divertrx}{'removed'}) {
-            tag 'orphaned-diversion', $divert, $script;
+            $self->tag('orphaned-diversion', $divert, $script);
         }
 
         # Handle man page diversions somewhat specially.  We may
@@ -1256,11 +1320,14 @@ sub binary {
         }
 
         if ($expand_diversions) {
-            tag 'diversion-for-unknown-file', $divert, "$script:$line"
-              unless (any { $_ =~ m/$divertrx/ } $info->sorted_index);
+            $self->tag('diversion-for-unknown-file', $divert, "$script:$line")
+              unless (
+                any { $_ =~ m/$divertrx/ }
+                $processable->installed->sorted_list
+              );
         } else {
-            tag 'diversion-for-unknown-file', $divert, "$script:$line"
-              unless $info->index($divert);
+            $self->tag('diversion-for-unknown-file', $divert, "$script:$line")
+              unless $processable->installed->lookup($divert);
         }
     }
 
@@ -1271,8 +1338,11 @@ sub binary {
 
 # try generic bad maintainer script command tagging
 sub generic_check_bad_command {
-    my ($line, $file, $lineno, $pkg, $findincatstring, $in_automatic_section)
+    my ($self, $line, $file, $lineno, $findincatstring, $in_automatic_section)
       = @_;
+
+    my $pkg = $self->package;
+
     # try generic bad maintainer script command tagging
   BAD_CMD:
     foreach my $bad_cmd_tag ($BAD_MAINT_CMD->all) {
@@ -1292,7 +1362,7 @@ sub generic_check_bad_command {
                 my $extrainfo = defined($1) ? "\'$1\'" : '';
                 my $inpackage = $bad_cmd_data->{'in_package'};
                 unless($pkg =~ m{$inpackage}) {
-                    tag $bad_cmd_tag, "$file:$lineno", $extrainfo;
+                    $self->tag($bad_cmd_tag, "$file:$lineno", $extrainfo);
                 }
             }
         }
@@ -1308,7 +1378,7 @@ sub script_is_evil_and_wrong {
     my $i = 0;
     my $var = '0';
     my $backgrounded = 0;
-    my $fd = $path->open;
+    open(my $fd, '<', $path->unpacked_path);
     local $_;
     while (<$fd>) {
         chomp;
@@ -1377,7 +1447,7 @@ sub script_is_evil_and_wrong {
 sub check_script_syntax {
     my ($interpreter, $path) = @_;
 
-    safe_qx($interpreter, '-n', $path->fs_path);
+    safe_qx($interpreter, '-n', $path->unpacked_path);
     return $?;
 }
 
@@ -1424,8 +1494,7 @@ sub _parse_interpreters {
     if ($dep eq '@NODEPS@') {
         $dep = '';
     } elsif ($dep =~ m/@/) {
-        internal_error(
-            "Unknown magic value $dep for versioned interpreter $interpreter");
+        die "Unknown magic value $dep for versioned interpreter $interpreter";
     }
     return [$path, $dep];
 }
@@ -1440,10 +1509,8 @@ sub _parse_versioned_interpreters {
     } elsif ($deprel eq '@SKIP_UNVERSIONED@') {
         $deprel = undef;
     } elsif ($deprel =~ m/@/) {
-        internal_error(
-            join(q{ },
-                "Unknown magic value $deprel",
-                "for versioned interpreter $interpreter"));
+        die
+          "Unknown magic value $deprel for versioned interpreter $interpreter";
     }
     return [$path, $deprel, qr/^$regex$/, $deptmp, \@versions];
 }
