@@ -19,8 +19,10 @@
 # MA 02110-1301, USA.
 
 package Lintian::shared_libs;
-use strict;
+
+use v5.20;
 use warnings;
+use utf8;
 use autodie;
 
 use File::Basename;
@@ -28,8 +30,12 @@ use List::MoreUtils qw(any none uniq);
 
 use Lintian::Data;
 use Lintian::Relation;
-use Lintian::Tags qw(tag);
-use Lintian::Util qw(internal_error strip);
+use Lintian::Util qw(strip);
+
+use Moo;
+use namespace::clean;
+
+with 'Lintian::Check';
 
 # Libraries that should only be used in the presence of certain capabilities
 # may be located in subdirectories of the standard ldconfig search path with
@@ -48,14 +54,20 @@ my %symbols_meta_fields = map { $_ => 1 }qw(
 my $ldconfig_dirs = Lintian::Data->new('shared-libs/ldconfig-dirs');
 my $MA_DIRS = Lintian::Data->new('common/multiarch-dirs', qr/\s++/);
 
-sub always {
-    my ($pkg, $type, $info, $proc, $group) = @_;
+sub installable {
+    my ($self) = @_;
 
-    my ($must_call_ldconfig, %SONAME, %SONAMES, %STATIC_LIBS, %sharedobject);
+    my $pkg = $self->package;
+    my $type = $self->type;
+    my $processable = $self->processable;
+    my $group = $self->group;
+
+    my ($must_call_ldconfig, %SONAME, %SHARED_LIB_PRESENT,
+        %STATIC_LIB_PRESENT, %sharedobject);
     my @shlibs;
     my @words;
     my @devpkgs;
-    my $objdump = $info->objdump_info;
+    my $objdump = $processable->objdump_info;
 
     # 1st step: get info about shared libraries installed by this package
     foreach my $file (sort keys %{$objdump}) {
@@ -63,9 +75,10 @@ sub always {
           if exists($objdump->{$file}{SONAME});
     }
 
-    foreach my $file ($info->sorted_index) {
-        $SONAMES{$1} = 1 if $file =~ m,.*\/lib([^/]+)\.so$,;
-        $STATIC_LIBS{$1} = 1 if $file =~ m,.*\/lib([^/]+)\.a$,;
+    foreach my $file ($processable->installed->sorted_list) {
+        $SHARED_LIB_PRESENT{$1} = 1
+          if $file =~ m%.*\/lib([^/]+)\.so(?:\.\d+){0,3}$%;
+        $STATIC_LIB_PRESENT{$1} = 1 if $file =~ m,.*\/lib([^/]+)\.a$,;
         next if not $file->is_file;
         my $fileinfo = $file->file_info;
         if (   $fileinfo =~ m/^[^,]*\bELF\b/
@@ -82,8 +95,8 @@ sub always {
 
     if (%SONAME) {
         foreach my $bin ($group->get_binary_processables) {
-            next unless $bin->pkg_name =~ m/\-dev$/;
-            if ($bin->info->relation('strong')->implies($pkg)) {
+            next unless $bin->name =~ m/\-dev$/;
+            if ($bin->relation('strong')->implies($pkg)) {
                 push @devpkgs, $bin;
             }
         }
@@ -91,12 +104,12 @@ sub always {
 
     # 2nd step: read package contents
 
-    for my $cur_file ($info->sorted_index) {
+    for my $cur_file ($processable->installed->sorted_list) {
         # shared library?
 
         my $normalized_target;
         $normalized_target = $cur_file->link_normalized
-          if defined($cur_file->link);
+          if length $cur_file->link;
 
         if (
             exists $SONAME{$cur_file}
@@ -127,7 +140,7 @@ sub always {
             # Now that we're sure this is really a shared library, report on
             # non-PIC problems.
             if ($objdump->{$cur_file}{TEXTREL}) {
-                tag 'shlib-with-non-pic-code', $cur_file;
+                $self->tag('shlib-with-non-pic-code', $cur_file);
             }
 
             my @symbol_names
@@ -137,7 +150,7 @@ sub always {
                 # If it has an INTERP section it might be an application with
                 # a SONAME (hi openjdk-6, see #614305).  Also see the comment
                 # for "shlib-with-executable-bit" below.
-                tag 'shlib-calls-exit', $cur_file
+                $self->tag('shlib-calls-exit', $cur_file)
                   unless $objdump->{$cur_file}{INTERP};
             }
 
@@ -148,28 +161,29 @@ sub always {
                 # designed to do something useful when executed, so don't
                 # report an error.  Also give ld.so a pass, since it's
                 # special.
-                tag 'shlib-with-executable-bit', $cur_file, $perms
+                $self->tag('shlib-with-executable-bit', $cur_file, $perms)
                   unless ($objdump->{$cur_file}{INTERP}
                     or $cur_file =~ m,^lib.*/ld-[\d.]+\.so$,);
             } elsif ($perm != 0644) {
-                tag 'shlib-with-bad-permissions', $cur_file, $perms;
+                $self->tag('shlib-with-bad-permissions', $cur_file, $perms);
             }
 
             # executable stack.
             if (not defined $objdump->{$cur_file}{'PH'}{STACK}) {
-                if (defined $info->field('architecture')) {
-                    my $arch = $info->field('architecture');
-                    tag 'shlib-without-PT_GNU_STACK-section', $cur_file;
+                if (defined $processable->field('architecture')) {
+                    my $arch = $processable->field('architecture');
+                    $self->tag('shlib-without-PT_GNU_STACK-section',$cur_file);
                 }
             } elsif ($objdump->{$cur_file}{'PH'}{STACK}{flags} ne 'rw-'){
-                tag 'shlib-with-executable-stack', $cur_file;
+                $self->tag('shlib-with-executable-stack', $cur_file);
             }
         } elsif ($ldconfig_dirs->known(dirname($cur_file))
             && exists $sharedobject{$cur_file}) {
-            tag 'sharedobject-in-library-directory-missing-soname', $cur_file;
-        } elsif ($cur_file =~ m/\.la$/ and not defined $cur_file->link) {
+            $self->tag('sharedobject-in-library-directory-missing-soname',
+                $cur_file);
+        } elsif ($cur_file =~ m/\.la$/ and not length $cur_file->link) {
             local $_;
-            my $fd = $cur_file->open;
+            open(my $fd, '<', $cur_file->unpacked_path);
             while(<$fd>) {
                 next
                   unless (m/^(libdir)='(.+?)'$/)
@@ -186,11 +200,12 @@ sub always {
                       if ($value
                         =~ m,^/usr/lib/python[\d.]+/(?:site|dist)-packages,
                         and $expected =~ m,^/usr/share/pyshared,);
-                    tag 'incorrect-libdir-in-la-file', $cur_file,
-                      "$value != $expected"
+                    $self->tag('incorrect-libdir-in-la-file', $cur_file,
+                        "$value != $expected")
                       unless($expected eq $value);
                 } elsif ($field eq 'dependency_libs'){
-                    tag 'non-empty-dependency_libs-in-la-file', $cur_file;
+                    $self->tag('non-empty-dependency_libs-in-la-file',
+                        $cur_file);
                 }
             }
             close($fd);
@@ -200,9 +215,8 @@ sub always {
     # 3rd step: check if shlib symlinks are present and in correct order
     for my $shlib_file (keys %SONAME) {
         # file found?
-        if (not $info->index($shlib_file)) {
-            internal_error(
-                "shlib $shlib_file not found in package (should not happen!)");
+        if (not $processable->installed->lookup($shlib_file)) {
+            die "shlib $shlib_file not found in package (should not happen!)";
         }
 
         my ($dir, $shlib_name) = $shlib_file =~ m,(.*)/([^/]+)$,;
@@ -212,31 +226,34 @@ sub always {
 
         # symlink found?
         my $link_file = "$dir/$SONAME{$shlib_file}";
-        if (not $info->index($link_file)) {
-            tag 'ldconfig-symlink-missing-for-shlib',
-              "$link_file $shlib_file $SONAME{$shlib_file}";
+        if (not $processable->installed->lookup($link_file)) {
+            $self->tag('ldconfig-symlink-missing-for-shlib',
+                "$link_file $shlib_file $SONAME{$shlib_file}");
         } else {
             # $link_file really another file?
             if ($link_file eq $shlib_file) {
                 # the library file uses its SONAME, this is ok...
             } else {
                 # $link_file really a symlink?
-                if ($info->index($link_file)->is_symlink) {
+                if ($processable->installed->lookup($link_file)->is_symlink) {
                     # yes.
 
                     # $link_file pointing to correct file?
-                    if ($info->index($link_file)->link eq $shlib_name) {
+                    if ($processable->installed->lookup($link_file)->link eq
+                        $shlib_name) {
                         # ok.
                     } else {
-                        tag 'ldconfig-symlink-referencing-wrong-file',
-                          join(q{ },
-                            "$link_file ->",
-                            $info->index($link_file)->link,
-                            "instead of $shlib_name");
+                        $self->tag(
+                            'ldconfig-symlink-referencing-wrong-file',
+                            join(q{ },
+                                "$link_file ->",
+                                $processable->installed->lookup($link_file)
+                                  ->link,
+                                "instead of $shlib_name"));
                     }
                 } else {
-                    tag 'ldconfig-symlink-is-not-a-symlink',
-                      "$shlib_file $link_file";
+                    $self->tag('ldconfig-symlink-is-not-a-symlink',
+                        "$shlib_file $link_file");
                 }
             }
         }
@@ -250,8 +267,10 @@ sub always {
         # if shlib doesn't _have_ a version, then $link_file and
         # $shlib_file will be equal, and it's not a development link,
         # so don't complain.
-        if ($info->index($link_file) and $link_file ne $shlib_file) {
-            tag 'non-dev-pkg-with-shlib-symlink', "$shlib_file $link_file";
+        if (    $processable->installed->lookup($link_file)
+            and $link_file ne $shlib_file) {
+            $self->tag('non-dev-pkg-with-shlib-symlink',
+                "$shlib_file $link_file");
         } elsif (@devpkgs) {
             # -dev package - it needs a shlib symlink
             my $ok = 0;
@@ -263,7 +282,7 @@ sub always {
 
             push @alt, $link_file;
 
-            if ($proc->pkg_src =~ m/^gcc-(\d+(?:.\d+)?)$/o) {
+            if ($processable->source =~ m/^gcc-(\d+(?:.\d+)?)$/o) {
                 # gcc has a lot of bi-arch libs and puts the dev symlink
                 # in slightly different directories (to be co-installable
                 # with itself I guess).  Allegedly, clang (etc.) have to
@@ -271,7 +290,7 @@ sub always {
                 # acceptable...
                 my $gcc_ver = $1;
                 my $basename = basename($link_file);
-                my $madir = $MA_DIRS->value($proc->pkg_arch);
+                my $madir = $MA_DIRS->value($processable->architecture);
                 my @madirs;
                 if (defined $madir) {
                     # For i386-*, the triplet GCC uses can be i586-* or i686-*.
@@ -307,31 +326,33 @@ sub always {
 
           PKG:
             foreach my $devpkg (@devpkgs) {
-                my $dinfo = $devpkg->info;
+
                 foreach my $link (@alt) {
-                    if ($dinfo->index($link)) {
+                    if ($devpkg->installed->lookup($link)) {
                         $ok = 1;
                         last PKG;
                     }
                 }
             }
-            tag 'dev-pkg-without-shlib-symlink', "$shlib_file $link_file"
+            $self->tag('dev-pkg-without-shlib-symlink',
+                "$shlib_file $link_file")
               unless $ok;
         }
     }
 
     # 4th step: check shlibs control file
     # $version may be undef in very broken packages
-    my $version = $info->field('version');
+    my $version = $processable->field('version');
     my $provides = $pkg;
     $provides .= "( = $version)" if defined $version;
     # Assume the version to be a non-native version to avoid
     # uninitialization warnings later.
     $version = '0-1' unless defined $version;
-    $provides = Lintian::Relation->and($info->relation('provides'), $provides);
+    $provides
+      = Lintian::Relation->and($processable->relation('provides'), $provides);
 
-    my $shlibsf = $info->control_index('shlibs');
-    my $symbolsf = $info->control_index('symbols');
+    my $shlibsf = $processable->control->lookup('shlibs');
+    my $symbolsf = $processable->control->lookup('symbols');
     my (%shlibs_control, %symbols_control);
 
     # control files are not symlinks (or other "weird" things).
@@ -348,7 +369,7 @@ sub always {
         my $soname = format_soname($SONAME{$_});
         if ($soname !~ / /) {
             $unversioned_shlibs{$_} = 1;
-            tag 'shlib-without-versioned-soname', $_, $soname
+            $self->tag('shlib-without-versioned-soname', $_, $soname)
               if $ldconfig_dirs->known(dirname($_));
         }
     }
@@ -358,7 +379,8 @@ sub always {
         # no shared libraries included in package, thus shlibs control
         # file should not be present
         if ($shlibsf) {
-            tag 'pkg-has-shlibs-control-file-but-no-actual-shared-libs';
+            $self->tag(
+                'pkg-has-shlibs-control-file-but-no-actual-shared-libs');
         }
     } else {
         # shared libraries included, thus shlibs control file has to exist
@@ -367,13 +389,13 @@ sub always {
                 for my $shlib (@shlibs) {
                     # skip it if it's not a public shared library
                     next unless $ldconfig_dirs->known(dirname($shlib));
-                    tag 'no-shlibs-control-file', $shlib
+                    $self->tag('no-shlibs-control-file', $shlib)
                       unless is_nss_plugin($shlib);
                 }
             }
         } elsif ($shlibsf->is_open_ok) {
             my (%shlibs_control_used, @shlibs_depends);
-            my $fd = $shlibsf->open;
+            open(my $fd, '<', $shlibsf->unpacked_path);
             while (<$fd>) {
                 chop;
                 next if m/^\s*$/ or /^#/;
@@ -386,8 +408,8 @@ sub always {
                 @words = split(/\s+/o,$_);
                 my $shlibs_string = $udeb.$words[0].' '.$words[1];
                 if ($shlibs_control{$shlibs_string}) {
-                    tag 'duplicate-entry-in-shlibs-control-file',
-                      $shlibs_string;
+                    $self->tag('duplicate-entry-in-shlibs-control-file',
+                        $shlibs_string);
                 } else {
                     $shlibs_control{$shlibs_string} = 1;
                     push(@shlibs_depends, join(' ', @words[2 .. $#words]))
@@ -404,13 +426,13 @@ sub always {
                     # skip it if it's not a public shared library
                     next unless $ldconfig_dirs->known(dirname($shlib));
                     # no!!
-                    tag 'shlib-missing-in-control-file', $shlib_name, 'for',
-                      $shlib
+                    $self->tag('shlib-missing-in-control-file',
+                        $shlib_name, 'for',$shlib)
                       unless is_nss_plugin($shlib);
                 }
             }
             for my $shlib_name (keys %shlibs_control) {
-                tag 'unused-shlib-entry-in-control-file', $shlib_name
+                $self->tag('unused-shlib-entry-in-control-file', $shlib_name)
                   unless $shlibs_control_used{$shlib_name};
             }
 
@@ -425,9 +447,10 @@ sub always {
             @shlibs_depends = uniq(@shlibs_depends);
             for my $depend (@shlibs_depends) {
                 unless ($provides->implies($depend)) {
-                    tag 'shlibs-declares-dependency-on-other-package', $depend;
+                    $self->tag('shlibs-declares-dependency-on-other-package',
+                        $depend);
                 }
-                tag 'shlibs-uses-obsolete-relation', $depend
+                $self->tag('shlibs-uses-obsolete-relation', $depend)
                   if $depend =~ m/\(\s*[><](?![<>=])\s*/;
             }
         }
@@ -439,7 +462,7 @@ sub always {
         # no shared libraries included in package, thus symbols
         # control file should not be present
         if ($symbolsf) {
-            tag 'pkg-has-symbols-control-file-but-no-shared-libs';
+            $self->tag('pkg-has-symbols-control-file-but-no-shared-libs');
         }
     } elsif (not $symbolsf) {
         if ($type ne 'udeb') {
@@ -450,7 +473,7 @@ sub always {
                 # appear in the symbol table
                 next if any { @{$_}[2] =~ m/^__objc_/ }
                 @{$objdump->{$shlib}{SYMBOLS}};
-                tag 'no-symbols-control-file', $shlib
+                $self->tag('no-symbols-control-file', $shlib)
                   unless is_nss_plugin($shlib);
             }
         }
@@ -468,7 +491,7 @@ sub always {
         my $warned = 0;
         my $symbol_count = 0;
 
-        my $fd = $symbolsf->open;
+        open(my $fd, '<', $symbolsf->unpacked_path);
         while (<$fd>) {
             chomp;
             next if m/^\s*$/ or /^#/;
@@ -481,7 +504,8 @@ sub always {
                 $soname = format_soname($soname);
 
                 if ($symbols_control{$soname}) {
-                    tag 'duplicate-entry-in-symbols-control-file', $soname;
+                    $self->tag('duplicate-entry-in-symbols-control-file',
+                        $soname);
                 } else {
                     $symbols_control{$soname} = 1;
                     $warned = 0;
@@ -494,7 +518,7 @@ sub always {
                             if (defined $dep_package) {
                                 push @symbols_depends, $dep_package . $dep;
                             } else {
-                                tag 'syntax-error-in-symbols-file', $.
+                                $self->tag('syntax-error-in-symbols-file', $.)
                                   unless $warned;
                                 $warned = 1;
                             }
@@ -511,7 +535,7 @@ sub always {
                 $warned = 0;
 
                 if (%meta_info_seen or not defined $soname) {
-                    tag 'syntax-error-in-symbols-file', $.;
+                    $self->tag('syntax-error-in-symbols-file', $.);
                     $warned = 1;
                 }
 
@@ -524,7 +548,7 @@ sub always {
                         if (defined $dep_package) {
                             push @symbols_depends, $dep_package . $dep;
                         } else {
-                            tag 'syntax-error-in-symbols-file', $.
+                            $self->tag('syntax-error-in-symbols-file', $.)
                               unless $warned;
                             $warned = 1;
                         }
@@ -535,9 +559,9 @@ sub always {
             } elsif (m/^\*\s(\S+):\s\S+/) {
                 # meta-information
 
-                tag 'unknown-meta-field-in-symbols-file', "$1, line $."
+                $self->tag('unknown-meta-field-in-symbols-file', "$1, line $.")
                   unless exists $symbols_meta_fields{$1};
-                tag 'syntax-error-in-symbols-file', $.
+                $self->tag('syntax-error-in-symbols-file', $.)
                   unless defined $soname and $symbol_count == 0;
 
                 $meta_info_seen{$1} = 1;
@@ -545,7 +569,7 @@ sub always {
             } elsif (m/^\s+(\S+)\s(\S+)(?:\s(\S+(?:\s\S+)?))?$/) {
                 # Symbol definition
 
-                tag 'syntax-error-in-symbols-file', $.
+                $self->tag('syntax-error-in-symbols-file', $.)
                   unless defined $soname;
 
                 $symbol_count++;
@@ -564,13 +588,13 @@ sub always {
 
                 if (length $dep_order) {
                     if ($dep_order !~ /^\d+$/ or $dep_order > $dep_templates) {
-                        tag 'invalid-template-id-in-symbols-file', $.;
+                        $self->tag('invalid-template-id-in-symbols-file', $.);
                     }
                 }
             } else {
                 # Unparseable line
 
-                tag 'syntax-error-in-symbols-file', $.;
+                $self->tag('syntax-error-in-symbols-file', $.);
             }
         }
         close($fd);
@@ -580,8 +604,9 @@ sub always {
             if ($full_version_count > 0) {
                 $others = " and $full_version_count others";
             }
-            tag 'symbols-file-contains-current-version-with-debian-revision',
-              "on symbol $full_version_sym$others";
+            $self->tag(
+                'symbols-file-contains-current-version-with-debian-revision',
+                "on symbol $full_version_sym$others");
         }
         if ($debian_revision_count) {
             $debian_revision_count--;
@@ -589,8 +614,10 @@ sub always {
             if ($debian_revision_count > 0) {
                 $others = " and $debian_revision_count others";
             }
-            tag 'symbols-file-contains-debian-revision',
-              "on symbol $debian_revision_sym$others";
+            $self->tag(
+                'symbols-file-contains-debian-revision',
+                "on symbol $debian_revision_sym$others"
+            );
         }
         for my $shlib (@shlibs, keys %unversioned_shlibs) {
             my $shlib_name = $SONAME{$shlib};
@@ -600,16 +627,17 @@ sub always {
             unless (exists $symbols_control{$shlib_name}) {
                 # skip it if it's not a public shared library
                 next unless $ldconfig_dirs->known(dirname($shlib));
-                tag 'shlib-missing-in-symbols-control-file', $shlib_name,
-                  'for', $shlib
+                $self->tag('shlib-missing-in-symbols-control-file',
+                    $shlib_name,'for', $shlib)
                   unless is_nss_plugin($shlib);
             }
         }
         for my $shlib_name (keys %symbols_control) {
-            tag 'unused-shlib-entry-in-symbols-control-file', $shlib_name
+            $self->tag('unused-shlib-entry-in-symbols-control-file',
+                $shlib_name)
               unless $symbols_control_used{$shlib_name};
         }
-        tag 'symbols-file-missing-build-depends-package-field'
+        $self->tag('symbols-file-missing-build-depends-package-field')
           unless $build_depends_seen;
 
         # Check that all of the packages listed as dependencies in the symbols
@@ -624,7 +652,8 @@ sub always {
             my $d = $depend;
             $d =~ s/ \#MINVER\#$//;
             unless ($provides->implies($d)) {
-                tag 'symbols-declares-dependency-on-other-package', $depend;
+                $self->tag('symbols-declares-dependency-on-other-package',
+                    $depend);
             }
         }
     }
@@ -635,43 +664,44 @@ sub always {
     if (keys %shlibs_control and keys %symbols_control) {
         for my $key (keys %symbols_control) {
             unless (exists $shlibs_control{$key} or $key !~ / /) {
-                tag 'symbols-declared-but-not-shlib', $key;
+                $self->tag('symbols-declared-but-not-shlib', $key);
             }
         }
     }
 
     # 6th step: check pre- and post- control files
-    if (my $preinst = $info->control_index_resolved_path('preinst')) {
+    if (my $preinst = $processable->control->resolve_path('preinst')) {
         if ($preinst->is_open_ok) {
-            if ($preinst->file_contents =~ m/^[^\#]*\bldconfig\b/m) {
-                tag 'maintscript-calls-ldconfig', 'preinst'
-                  # Assume it is needed if glibc does it
-                  if $proc->pkg_src ne 'glibc';
+            if ($preinst->slurp =~ m/^[^\#]*\bldconfig\b/m) {
+                $self->tag(
+                    'maintscript-calls-ldconfig', 'preinst'
+                      # Assume it is needed if glibc does it
+                ) if $processable->source ne 'glibc';
             }
         }
     }
 
     my $we_trigger_ldconfig = 0;
-    if (my $postinst = $info->control_index_resolved_path('postinst')) {
+    if (my $postinst = $processable->control->resolve_path('postinst')) {
         if ($postinst->is_open_ok) {
             # Decide if we call ldconfig
-            if ($postinst->file_contents =~ m/^[^\#]*\bldconfig\b/m) {
+            if ($postinst->slurp =~ m/^[^\#]*\bldconfig\b/m) {
                 if ($type eq 'udeb') {
-                    tag 'udeb-postinst-must-not-call-ldconfig';
+                    $self->tag('udeb-postinst-must-not-call-ldconfig');
                 } else {
                     # glibc (notably libc-bin) needs to call ldconfig in
                     # order to implement the "ldconfig" trigger.
-                    tag 'maintscript-calls-ldconfig', 'postinst'
-                      if $proc->pkg_src ne 'glibc';
+                    $self->tag('maintscript-calls-ldconfig', 'postinst')
+                      if $processable->source ne 'glibc';
                 }
             }
         }
     }
 
-    if (my $triggers = $info->control_index_resolved_path('triggers')) {
+    if (my $triggers = $processable->control->resolve_path('triggers')) {
         if ($triggers->is_open_ok) {
             # Determine if the package had an ldconfig trigger
-            my $fd = $triggers->open;
+            open(my $fd, '<', $triggers->unpacked_path);
             while (my $line = <$fd>) {
                 strip($line);
                 $line =~ tr/ \t/ /s;
@@ -685,61 +715,46 @@ sub always {
     }
 
     if ($type eq 'udeb') {
-        tag 'udeb-postinst-must-not-call-ldconfig'
+        $self->tag('udeb-postinst-must-not-call-ldconfig')
           if $we_trigger_ldconfig;
     } else {
-        tag 'package-has-unnecessary-activation-of-ldconfig-trigger'
+        $self->tag('package-has-unnecessary-activation-of-ldconfig-trigger')
           if $we_trigger_ldconfig and not $must_call_ldconfig;
-        tag 'package-must-activate-ldconfig-trigger', $must_call_ldconfig
+        $self->tag('package-must-activate-ldconfig-trigger',
+            $must_call_ldconfig)
           if not $we_trigger_ldconfig and $must_call_ldconfig;
     }
 
-    my $multiarch = $info->field('multi-arch', 'no');
+    my $multiarch = $processable->field('multi-arch', 'no');
     if ($multiarch eq 'foreign' and $must_call_ldconfig) {
-        tag 'shlib-in-multi-arch-foreign-package', $must_call_ldconfig;
+        $self->tag('shlib-in-multi-arch-foreign-package', $must_call_ldconfig);
     }
 
-    if (my $prerm = $info->control_index_resolved_path('prerm')) {
+    if (my $prerm = $processable->control->resolve_path('prerm')) {
         if ($prerm->is_open_ok) {
-            if ($prerm->file_contents =~ m/^[^\#]*\bldconfig\b/m) {
-                tag 'maintscript-calls-ldconfig', 'prerm'
-                  # Assume it is needed if glibc does it
-                  if $proc->pkg_src ne 'glibc';
+            if ($prerm->slurp =~ m/^[^\#]*\bldconfig\b/m) {
+                $self->tag(
+                    'maintscript-calls-ldconfig', 'prerm'
+                      # Assume it is needed if glibc does it
+                ) if $processable->source ne 'glibc';
             }
         }
     }
 
-    if (my $postrm = $info->control_index_resolved_path('postrm')) {
+    if (my $postrm = $processable->control->resolve_path('postrm')) {
         if ($postrm->is_open_ok) {
-            my $contents = $postrm->file_contents;
+            my $contents = $postrm->slurp;
 
             # Decide if we call ldconfig
             if ($contents =~ m/^[^\#]*\bldconfig\b/m) {
-                tag 'maintscript-calls-ldconfig', 'postrm'
-                  # Assume it is needed if glibc does it
-                  if $proc->pkg_src ne 'glibc';
+                $self->tag(
+                    'maintscript-calls-ldconfig', 'postrm'
+                      # Assume it is needed if glibc does it
+                ) if $processable->source ne 'glibc';
             }
         }
     }
 
-    foreach my $file ($info->sorted_index) {
-        next unless $file =~ m,^usr/(lib(/[^/]+)?|share)/pkgconfig/[^/]+\.pc$,;
-        next unless $file->is_open_ok;
-        my $fd = $file->open;
-        while (<$fd>) {
-            next unless m,^Libs:,;
-            while (/[:\s]-l(\S+)/g) {
-                tag 'pkg-config-references-unknown-shared-library',
-                  $file, "-l$1", "(line $.)"
-                  unless $1 =~ m/\$\{.+\}/
-                  or exists($SONAMES{$1})
-                  or exists($STATIC_LIBS{$1})
-                  or $UNKNOWN_SHARED_LIBRARY_EXCEPTIONS->known($1);
-            }
-        }
-        close($fd);
-    }
-
     return;
 }
 
@@ -779,7 +794,7 @@ sub needs_ldconfig {
     $dirname .= "/$last" if $last;
     # yes! so postinst must call ldconfig
     return 1 if $ldconfig_dirs->known($dirname);
-    return;
+    return 0;
 }
 
 1;
