@@ -1,7 +1,8 @@
 # Hey emacs! This is a -*- Perl -*- script!
 # Lintian::Util -- Perl utility functions for lintian
 
-# Copyright (C) 1998 Christian Schwarz
+# Copyright © 1998 Christian Schwarz
+# Copyright © 2020 Felix Lechner
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,14 +21,19 @@
 # MA 02110-1301, USA.
 
 package Lintian::Util;
-use strict;
+
+use v5.20;
 use warnings;
+use utf8;
 use autodie;
 
 use Carp qw(croak);
 use Cwd qw(abs_path);
 use Errno qw(ENOENT);
 use Exporter qw(import);
+use IO::Async::Loop;
+use IO::Async::Process;
+use Path::Tiny;
 use POSIX qw(sigprocmask SIG_BLOCK SIG_UNBLOCK SIG_SETMASK);
 
 use Lintian::Deb822Parser qw(read_dpkg_control parse_dpkg_control);
@@ -52,28 +58,26 @@ BEGIN {
           get_file_digest
           file_is_encoded_in_non_utf8
           is_string_utf8_encoded
-          fail
-          internal_error
           do_fork
           run_cmd
+          safe_qx
           strip
           lstrip
           rstrip
           copy_dir
+          human_bytes
           gunzip_file
           open_gz
+          gzip
           perm2oct
           check_path
           clean_env
           normalize_pkg_path
-          parse_boolean
           is_ancestor_of
           locate_helper_tool
           drain_pipe
-          signal_number2name
-          dequote_name
-          pipe_tee
-          untaint
+          drop_relative_prefix
+          read_md5sums
           $PKGNAME_REGEX
           $PKGREPACK_REGEX
           $PKGVERSION_REGEX
@@ -86,6 +90,32 @@ use Encode ();
 use FileHandle;
 use Scalar::Util qw(openhandle);
 
+use Lintian::Relation::Version qw(versions_equal versions_comparator);
+
+use constant EMPTY => q{};
+use constant SPACE => q{ };
+use constant COLON => q{:};
+use constant BACKSLASH => q{\\};
+use constant NEWLINE => qq{\n};
+
+# read up to 40kB at a time.  this happens to be 4096 "tar records"
+# (with a block-size of 512 and a block factor of 20, which appear to
+# be the defaults).  when we do full reads and writes of READ_SIZE (the
+# OS willing), the receiving end will never be with an incomplete
+# record.
+use constant READ_SIZE => 4096 * 20 * 512;
+
+# preload cache for common permission strings
+# call overhead o perm2oct was measurable on chromium-browser/32.0.1700.123-2
+# load time went from ~1.5s to ~0.1s; of 115363 paths, only 306 were uncached
+my %OCTAL_LOOKUP = map { $_ => perm2oct($_) } (
+    '-rw-r--r--', # standard (non-executable) file
+    '-rwxr-xr-x', # standard executable file
+    'drwxr-xr-x', # standard dir perm
+    'drwxr-sr-x', # standard dir perm with suid (lintian-lab on lintian.d.o)
+    'lrwxrwxrwx', # symlinks
+);
+
 =head1 NAME
 
 Lintian::Util - Lintian utility functions
@@ -122,7 +152,8 @@ our $PKGNAME_REGEX = qr/[a-z0-9][-+\.a-z0-9]+/o;
 =item $PKGREPACK_REGEX
 
 Regular expression that matches "repacked" package names.  The expression is
-not anchored and does not enforce any "boundary" characters.
+not anchored and does not enforce any "boundary" characters. It should only
+be applied to the upstream portion (see #931846).
 
 =cut
 
@@ -170,40 +201,99 @@ L</parse_dpkg_control> do.  It can also emit:
 
 =cut
 
-{
+sub get_deb_info {
+    my ($path) = @_;
 
-    my $loaded = 0;
+    # dpkg-deb -f $file is very slow. Instead, we use ar and tar.
 
-    sub get_deb_info {
-        my ($file) = @_;
+    my $loop = IO::Async::Loop->new;
 
-        # dpkg-deb -f $file is very slow. Instead, we use ar and tar.
-        my $opts = {
-            fail => 'exception',
-            pipe_out => FileHandle->new
-        };
+    # get control tarball from deb
+    my $dpkgerror;
+    my $dpkgfuture = $loop->new_future;
+    my @dpkgcommand = ('dpkg-deb', '--ctrl-tarfile', $path);
+    my $dpkgprocess = IO::Async::Process->new(
+        command => [@dpkgcommand],
+        stdout => { via => 'pipe_read' },
+        stderr => { into => \$dpkgerror },
+        on_finish => sub {
+            my ($self, $exitcode) = @_;
+            my $status = ($exitcode >> 8);
 
-        if (not $loaded) {
-            $loaded++;
-            require Lintian::Command;
-        }
+            if ($status) {
+                my $message= "Non-zero status $status from @dpkgcommand";
+                $message .= COLON . NEWLINE . $dpkgerror
+                  if length $dpkgerror;
+                $dpkgfuture->fail($message);
+                return;
+            }
 
-        Lintian::Command::spawn(
-            $opts, ['dpkg-deb', '--ctrl-tarfile', $file],
-            '|', ['tar', '--wildcards', '-xO', '-f', '-', '*control']);
-        my @data = parse_dpkg_control($opts->{pipe_out});
+            $dpkgfuture->done("Done with @dpkgcommand");
+            return;
+        });
 
-        # Consume all data before exiting so that we don't kill child processes
-        # with SIGPIPE.  This will normally only be an issue with malformed
-        # control files.
-        drain_pipe($opts->{pipe_out});
-        close($opts->{pipe_out});
-        $opts->{harness}->finish;
-        return $data[0];
-    }
+    my $control;
+
+    # get the control file
+    my $tarerror;
+    my $tarfuture = $loop->new_future;
+    my @tarcommand = ('tar', '--wildcards', '-xO', '-f', '-', '*control');
+    my $tarprocess = IO::Async::Process->new(
+        command => [@tarcommand],
+        stdin => { via => 'pipe_write' },
+        stdout => { into => \$control },
+        stderr => { into => \$tarerror },
+        on_finish => sub {
+            my ($self, $exitcode) = @_;
+            my $status = ($exitcode >> 8);
+
+            if ($status) {
+                my $message = "Non-zero status $status from @tarcommand";
+                $message .= COLON . NEWLINE . $tarerror
+                  if length $tarerror;
+                $tarfuture->fail($message);
+                return;
+            }
+
+            $tarfuture->done("Done with @tarcommand");
+            return;
+        });
+
+    $tarprocess->stdin->configure(write_len => READ_SIZE);
+
+    $dpkgprocess->stdout->configure(
+        read_len => READ_SIZE,
+        on_read => sub {
+            my ($stream, $buffref, $eof) = @_;
+
+            if (length $$buffref) {
+                $tarprocess->stdin->write($$buffref);
+                $$buffref = EMPTY;
+            }
+
+            if ($eof) {
+                $tarprocess->stdin->close_when_empty;
+            }
+
+            return 0;
+        },
+    );
+
+    $loop->add($dpkgprocess);
+    $loop->add($tarprocess);
+
+    # awaits, and dies on failure with message from failed constituent
+    my $composite = Future->needs_all($dpkgfuture, $tarfuture);
+    $composite->get;
+
+    open(my $fh, '<', \$control);
+    my @data = parse_dpkg_control($fh);
+    close $fh;
+
+    return $data[0];
 }
 
-=item get_dsc_control (DSCFILE)
+=item get_dsc_info (DSCFILE)
 
 Convenience function for reading dsc files.  It will read the DSCFILE
 using L</read_dpkg_control(FILE[, FLAGS[, LINES]])> and then return the
@@ -432,40 +522,46 @@ Examples:
 =cut
 
 sub perm2oct {
-    my ($t) = @_;
+    my ($text) = @_;
 
-    my $o = 0;
+    my $lookup = $OCTAL_LOOKUP{$text};
+    return $lookup
+      if defined $lookup;
+
+    my $octal = 0;
 
     # Types:
     #  file (-), block/character device (b & c), directory (d),
     #  hardlink (h), symlink (l), named pipe (p).
     if (
-        $t !~ m/^   [-bcdhlp]                # file type
+        $text !~ m/^   [-bcdhlp]                # file type
                     ([-r])([-w])([-xsS])     # user
                     ([-r])([-w])([-xsS])     # group
                     ([-r])([-w])([-xtT])     # other
-               /xsmo
+               /xsm
     ) {
-        croak "$t does not appear to be a permission string";
+        croak "$text does not appear to be a permission string";
     }
 
-    $o += 00400 if $1 eq 'r';   # owner read
-    $o += 00200 if $2 eq 'w';   # owner write
-    $o += 00100 if $3 eq 'x';   # owner execute
-    $o += 04000 if $3 eq 'S';   # setuid
-    $o += 04100 if $3 eq 's';   # setuid + owner execute
-    $o += 00040 if $4 eq 'r';   # group read
-    $o += 00020 if $5 eq 'w';   # group write
-    $o += 00010 if $6 eq 'x';   # group execute
-    $o += 02000 if $6 eq 'S';   # setgid
-    $o += 02010 if $6 eq 's';   # setgid + group execute
-    $o += 00004 if $7 eq 'r';   # other read
-    $o += 00002 if $8 eq 'w';   # other write
-    $o += 00001 if $9 eq 'x';   # other execute
-    $o += 01000 if $9 eq 'T';   # stickybit
-    $o += 01001 if $9 eq 't';   # stickybit + other execute
+    $octal += 00400 if $1 eq 'r';   # owner read
+    $octal += 00200 if $2 eq 'w';   # owner write
+    $octal += 00100 if $3 eq 'x';   # owner execute
+    $octal += 04000 if $3 eq 'S';   # setuid
+    $octal += 04100 if $3 eq 's';   # setuid + owner execute
+    $octal += 00040 if $4 eq 'r';   # group read
+    $octal += 00020 if $5 eq 'w';   # group write
+    $octal += 00010 if $6 eq 'x';   # group execute
+    $octal += 02000 if $6 eq 'S';   # setgid
+    $octal += 02010 if $6 eq 's';   # setgid + group execute
+    $octal += 00004 if $7 eq 'r';   # other read
+    $octal += 00002 if $8 eq 'w';   # other write
+    $octal += 00001 if $9 eq 'x';   # other execute
+    $octal += 01000 if $9 eq 'T';   # stickybit
+    $octal += 01001 if $9 eq 't';   # stickybit + other execute
 
-    return $o;
+    $OCTAL_LOOKUP{$text} = $octal;
+
+    return $octal;
 }
 
 =item run_cmd([OPTS, ]COMMAND[, ARGS...])
@@ -557,6 +653,54 @@ sub run_cmd {
     return 1;
 }
 
+=item C<safe_qx(@cmd)>
+
+Emulates the C<qx()> operator by returning the captured output
+just like Capture::Tiny;
+
+Examples:
+
+  # Capture the output of a simple command
+  my $output = safe_qx('grep', 'some-pattern', 'path/to/file');
+
+=cut
+
+sub safe_qx {
+    my @command = @_;
+
+    my $loop = IO::Async::Loop->new;
+    my $future = $loop->new_future;
+    my $status;
+
+    $loop->run_child(
+        command => [@command],
+        on_finish => sub {
+            my ($pid, $exitcode, $stdout, $stderr) = @_;
+            $status = ($exitcode >> 8);
+
+            if ($status) {
+                my $message = "Command @command exited with status $status";
+                $message .= ": $stderr" if length $stderr;
+                $future->fail($message);
+                return;
+            }
+
+            $future->done($stdout);
+        });
+
+    $loop->await($future);
+
+    if ($future->is_failed) {
+        $? = $status;
+        return $future->failure;
+    }
+
+    $? = 0;
+
+    # will raise an exception in case of failure
+    return $future->get;
+}
+
 =item copy_dir (ARGS)
 
 Convenient way of calling I<cp -a ARGS>.
@@ -570,6 +714,28 @@ sub copy_dir {
     return run_cmd('cp', '-a', '--reflink=auto', '--', @_);
 }
 
+=item human_bytes(SIZE)
+
+=cut
+
+sub human_bytes {
+    my ($size) = @_;
+
+    my @units = ('B', 'kiB', 'MiB', 'GiB');
+
+    my $unit = shift @units;
+
+    while ($size > 1536 && @units) {
+
+        $size /= 1024;
+        $unit = shift @units;
+    }
+
+    my $human = sprintf('%.0f %s', $size, $unit);
+
+    return $human;
+}
+
 =item gunzip_file (IN, OUT)
 
 Decompresses contents of the file IN and stores the contents in the
@@ -579,8 +745,32 @@ will cause a trappable error.
 =cut
 
 sub gunzip_file {
-    my ($in, $out) = @_;
-    run_cmd({out => $out}, 'gzip', '-dc', $in);
+    my ($inpath, $outpath) = @_;
+
+    my $loop = IO::Async::Loop->new;
+    my $future = $loop->new_future;
+
+    my @command = ('gzip', '--decompress', '--stdout', $inpath);
+    $loop->run_child(
+        command => [@command],
+        on_finish => sub {
+            my ($pid, $exitcode, $stdout, $stderr) = @_;
+            my $status = ($exitcode >> 8);
+
+            if ($status) {
+                my $message = "Command @command exited with status $status";
+                $message .= ": $stderr" if length $stderr;
+                $future->fail($message);
+                return;
+            }
+
+            path($outpath)->spew($stdout);
+            $future->done;
+        });
+
+    # will raise an exception in case of failure
+    $future->get;
+
     return;
 }
 
@@ -609,39 +799,53 @@ sub __open_gz_ext {
     return $fd;
 }
 
-=item internal_error (MSG[, ...])
+=item gzip (DATA, PATH)
 
-Use to signal an internal error. The argument(s) will used to print a
-diagnostic message to the user.
-
-If multiple arguments are given, they will be merged into a single
-string (by join (' ', @_)).  If only one argument is given it will be
-stringified and used directly.
-
-=item fail (MSG[, ...])
-
-Deprecated alias of "internal_error".
+Compresses DATA using gzip and stores result in file located at PATH.
 
 =cut
 
-sub fail {
-    warnings::warnif('deprecated',
-        '[deprecation] fail() has been replaced by internal_error()');
-    goto \&internal_error;
-}
+sub gzip {
+    my ($data, $path) = @_;
 
-sub internal_error {
-    my $str = 'internal error: ';
-    if (@_) {
-        $str .= join ' ', @_;
-    } else {
-        if ($!) {
-            $str .= "$!";
-        } else {
-            $str .= 'No context.';
-        }
-    }
-    croak $str;
+    unlink($path)
+      if -e $path;
+
+    $data //= EMPTY;
+
+    my $loop = IO::Async::Loop->new;
+    my $future = $loop->new_future;
+    my $compressed;
+    my $stderr;
+
+    my @command = ('gzip', '--best', '--no-name', '--stdout');
+    my $process = IO::Async::Process->new(
+        command => [@command],
+        stdin => { from => $data },
+        stdout => { into => \$compressed },
+        stderr => { into => \$stderr },
+        on_finish => sub {
+            my ($self, $exitcode) = @_;
+            my $status = ($exitcode >> 8);
+
+            if ($status) {
+                my $message = "Command @command exited with status $status";
+                $message .= ": $stderr" if length $stderr;
+                $future->fail($message);
+                return;
+            }
+
+            $future->done('Done with command @command');
+            return;
+        });
+
+    $loop->add($process);
+
+    $future->get;
+
+    path($path)->spew($compressed // EMPTY);
+
+    return;
 }
 
 =item locate_helper_tool(TOOLNAME)
@@ -664,7 +868,7 @@ If the tool cannot be found, this sub will cause a trappable error.
     sub locate_helper_tool {
         my ($toolname) = @_;
         if ($toolname =~ m{(?:\A|/) \.\. (?:\Z|/)}xsm) {
-            internal_error("$toolname is not a valid tool name");
+            croak "$toolname is not a valid tool name";
         }
         return $_CACHE{$toolname} if exists $_CACHE{$toolname};
 
@@ -681,11 +885,8 @@ If the tool cannot be found, this sub will cause a trappable error.
             }
         }
         $toolpath_str //= '<N/A>';
-        internal_error(
-            sprintf(
-                'Cannot locate %s (search dirs: %s)',
-                $toolname, $toolpath_str
-            ));
+
+        croak "Cannot locate $toolname (search dirs: $toolpath_str)";
     }
 }
 
@@ -772,26 +973,19 @@ sub check_path {
     return 0;
 }
 
-=item dequote_name(STR, REMOVESLASH)
+=item drop_relative_prefix(STRING)
 
-Strip an extra layer quoting in index file names and optionally
-remove an initial "./" if any.
-
-Remove initial ./ by default
+Remove an initial ./ from STRING, if present
 
 =cut
 
-sub dequote_name {
-    my ($name, $slsd) = @_;
-    $slsd = 1 unless defined $slsd;
-    $name =~ s,^\.?/,, if $slsd;
-    # Optimise for the case where the filename does not contain
-    # backslashes.  It is a fairly rare to see that in practise.
-    if (index($name, '\\') > -1) {
-        $name =~ s/(\G|[^\\](?:\\\\)*)\\(\d{3})/"$1" . chr(oct $2)/ge;
-        $name =~ s/\\\\/\\/g;
-    }
-    return $name;
+sub drop_relative_prefix {
+    my ($name) = @_;
+
+    my $copy = $name;
+    $copy =~ s{^\./}{}s;
+
+    return $copy;
 }
 
 =item signal_number2name(NUM)
@@ -909,7 +1103,7 @@ sub normalize_pkg_path {
     #
     # Note that @normalised will be empty if we end in the root
     # (e.g. '/' + 'usr' + '..' -> '/'), this is fine.
-    while (my $target = shift(@queue)) {
+    while (defined(my $target = shift(@queue))) {
         if ($target eq '..') {
             # are we out of bounds?
             return unless @normalised;
@@ -925,38 +1119,6 @@ sub normalize_pkg_path {
     return join('/', @normalised);
 }
 
-=item parse_boolean (STR)
-
-Attempt to parse STR as a boolean and return its value.
-If STR is not a valid/recognised boolean, the sub will
-invoke croak.
-
-The following values recognised (string checks are not
-case sensitive):
-
-=over 4
-
-=item The integer 0 is considered false
-
-=item Any non-zero integer is considered true
-
-=item "true", "y" and "yes" are considered true
-
-=item "false", "n" and "no" are considered false
-
-=back
-
-=cut
-
-sub parse_boolean {
-    my ($str) = @_;
-    return $str == 0 ? 0 : 1 if $str =~ m/^-?\d++$/o;
-    $str = lc $str;
-    return 1 if $str eq 'true' or $str =~ m/^y(?:es)?$/;
-    return 0 if $str eq 'false' or $str =~ m/^no?$/;
-    croak "\"$str\" is not a valid boolean value";
-}
-
 =item is_ancestor_of(PARENTDIR, PATH)
 
 Returns true if and only if PATH is PARENTDIR or a path stored
@@ -988,75 +1150,96 @@ sub is_ancestor_of {
     return 0;
 }
 
-=item pipe_tee(INHANDLE, OUTHANDLES[, OPTS])
+=item read_md5sums
 
-Read bytes from INHANDLE and copy them into all of the handles in the
-listref OUTHANDLES. The optional OPTS argument is a hashref of
-options, see below.
-
-The subroutine will continue to read from INHANDLE until it is
-exhausted or an error occurs (either during read or write).  In case
-of errors, a trappable error will be raised.  The handles are left
-open when the subroutine returns, caller must close them afterwards.
-
-Caller should ensure that handles are using "blocking" I/O.  The
-subroutine will use L<sysread|perlfunc/sysread> and
-L<syswrite|perlfunc/syswrite> when reading and writing.
-
-
-OPTS, if given, may contain the following key-value pairs:
-
-=over 4
-
-=item chunk_size
-
-A suggested buffer size for read/write.  If given, it will be to
-sysread as LENGTH argument when reading from INHANDLE.
-
-=back
+=item unescape_md5sum_filename
 
 =cut
 
-sub pipe_tee {
-    my ($in_fd, $out_ref, $opts) = @_;
-    my $read_size = ($opts && $opts->{'chunk_size'}) // 8096;
-    my @outs = @{$out_ref};
-    my $buffer;
-    while (1) {
-        # Disable autodie, because it includes the buffer
-        # exception.  Said buffer will get printed on errors
-        # yielding completely unreadable errors and a terminal
-        # drowned in binary characters.
-        no autodie qw(sysread syswrite);
-        my $rlen = sysread($in_fd, $buffer, $read_size);
-        if (not $rlen) {
-            last if defined($rlen);
-            croak("Failed to read from input handle: $!");
-        }
-        for my $out_fd (@outs) {
-            my $written = 0;
-            while ($written < $rlen) {
-                my $remain = $rlen - $written;
-                my $res = syswrite($out_fd, $buffer, $remain,$written);
-                if (!defined($res)) {
-                    croak("Failed to write to output handle: $!");
-                }
-                $written += $res;
-            }
+sub unescape_md5sum_filename {
+    my ($string, $problematic) = @_;
+
+    # done if there are no escapes
+    return $string
+      unless $problematic;
+
+    # split into individual characters
+    my @array = split(//, $string);
+
+# https://www.gnu.org/software/coreutils/manual/html_node/md5sum-invocation.html
+    my $path;
+    my $escaped = 0;
+    for my $char (@array) {
+
+        # start escape sequence
+        if ($char eq BACKSLASH && !$escaped) {
+            $escaped = 1;
+            next;
         }
+
+        # unescape newline
+        $char = NEWLINE
+          if $char eq 'n' && $escaped;
+
+        # append character
+        $path .= $char;
+
+        # end any escape sequence
+        $escaped = 0;
     }
-    return 1;
+
+    # do not stop inside an escape sequence
+    die 'Name terminated inside an escape sequence'
+      if $escaped;
+
+    return $path;
 }
 
-=item untaint(VALUE)
+sub read_md5sums {
+    my ($text) = @_;
 
-Untaint VALUE
+    # make a local copy
+    my $buffer = $text;
 
-=cut
+    my %checksums;
+    my @errors;
 
-sub untaint {
-    return $_[0] = $1 if $_[0] =~ m/^(.*)$/;
-    return;
+  TRY_AGAIN:
+
+    # start with checksum; processing style inspired by IO::Async::Stream
+    while ($buffer =~ s/^((?:\\)?\S{32}) [ *]//) {
+
+        my $checksum = $1;
+        my $problematic = 0;
+
+        # leading slash in checksum indicates an escaped name
+        $problematic = 1
+          if $checksum =~ s{^\\}{};
+
+        # read up until the next newline
+        $buffer =~ s/^([^\n]+)\n//;
+        my $string = $1;
+
+        my $path = unescape_md5sum_filename($string, $problematic);
+
+        push(@errors, "Empty name for checksum $checksum")
+          unless length $path;
+
+        $checksums{$path} = $checksum;
+    }
+
+    if ($buffer =~ s/([^\n]*)(?:\n|\z)//) {
+        my $unreadable = $1;
+
+        # this skips empty lines
+        push(@errors, "Odd text: $unreadable")
+          if length $unreadable;
+
+        goto TRY_AGAIN
+          if length $buffer;
+    }
+
+    return (\%checksums, \@errors);
 }
 
 =back
