@@ -29,9 +29,9 @@ Test::Lintian -- Check Lintian files for issues
  use Test::Lintian;
  use Test::More import => ['done_testing'];
  test_load_profiles('some/path');
- 
+
  done_testing;
- 
+
  # file 2
  use Test::Lintian;
  use Test::More import => ['done_testing'];
@@ -39,7 +39,7 @@ Test::Lintian -- Check Lintian files for issues
  test_check_desc('some/path/checks');
  test_load_checks('some/path/checks');
  test_tags_implemented('some/path/checks');
- 
+
  done_testing;
 
 =head1 DESCRIPTION
@@ -49,8 +49,9 @@ errors.
 
 =cut
 
-use strict;
+use v5.20;
 use warnings;
+use utf8;
 
 my $CLASS = __PACKAGE__;
 my $PROFILE;
@@ -70,22 +71,21 @@ use parent 'Test::Builder::Module';
 use Cwd qw(realpath);
 use File::Basename qw(basename);
 use File::Find ();
+use List::MoreUtils qw{any};
 use Path::Tiny;
 
-use Lintian::Check qw(check_spelling);
+use Lintian::Spelling qw(check_spelling);
 use Lintian::Data;
 use Lintian::Deb822Parser qw(read_dpkg_control);
 use Lintian::Profile;
 use Lintian::Tag::Info;
-use Lintian::Tags;
 use Lintian::Util qw(is_string_utf8_encoded);
 
 # We want data files loaded early to avoid problems with missing data
 # files ending up in releases (like in 2.5.17 and 2.5.18).
 $Lintian::Data::LAZY_LOAD = 0;
 
-my %severities = map { $_ => 1 } @Lintian::Tags::SEVERITIES;
-my %certainties = map { $_ => 1 } @Lintian::Tags::CERTAINTIES;
+my %severities = map { $_ => 1 } @Lintian::Tag::Info::SEVERITIES;
 my %check_types = map { $_ => 1 } qw(binary changes source udeb);
 my %known_html_tags = map { $_ => 1 } qw(a em i tt);
 
@@ -121,15 +121,6 @@ OPTS may contain the following key/value pairs:
 
 =over 4
 
-=item coll-dir
-
-Path to the collection directory (defaults to:
-/usr/share/lintian/collection).  This is mostly useful for testing
-Lintian itself.
-
-If set to C<undef>, the test of Needs-Info containing only existing
-collections will be skipped.
-
 =item filter
 
 If defined, it is a filter function that examines $_ (or its first
@@ -153,7 +144,7 @@ translations.  Otherwise, they must be regular checks.
 =cut
 
 sub test_check_desc {
-    my ($opts, @descs);
+    my ($opts, @dirs);
     my $builder = $CLASS->builder;
     my $colldir = '/usr/share/lintian/collection';
     my $find_opt = {'filter' => undef,};
@@ -161,14 +152,14 @@ sub test_check_desc {
 
     if (ref $_[0] eq 'HASH') {
         $opts = shift;
-        $colldir = $opts->{'coll-dir'}//'' if exists $opts->{'coll-dir'};
         $find_opt->{'filter'} = $opts->{'filter'} if exists $opts->{'filter'};
     }
     $opts //= {};
-    @descs = @_;
+    @dirs = @_;
     load_profile_for_test();
 
-    foreach my $desc_file (map { _find_check($find_opt, $_) } @descs) {
+    my @descs = map { _find_check($find_opt, $_) } @dirs;
+    foreach my $desc_file (@descs) {
         my ($header, @tagpara);
         eval {($header, @tagpara) = read_dpkg_control($desc_file);};
         if (my $err = $@) {
@@ -210,38 +201,9 @@ sub test_check_desc {
             );
         }
 
-        if ($needs and $colldir ne '') {
-            my @bad;
-            # new lines are not allowed, map them to "\\n" for readability.
-            $needs =~ s/\n/\\n/go;
-            foreach my $need (split m/\s*+,\s*+/o, $needs) {
-                push @bad, $need unless -f "$colldir/$need.desc";
-            }
-            $builder->is_eq(join(', ', @bad),
-                '', "$cname has unknown collections in Needs-Info");
-        } else {
-            $builder->ok(1, "$content_type has a valid Needs-Info (empty)")
-              if $colldir ne '';
-            $builder->skip(
-                'Needs-Info test checks skipped due to empty coll-dir')
-              if $needs ne '';
-        }
-        if (my $d = $header->{'info'}) {
-            my $mistakes = 0;
-            my $handler = sub {
-                my ($incorrect, $correct) = @_;
-                $builder->diag("Spelling ($cname): $incorrect => $correct");
-                $mistakes++;
-            };
-            check_spelling($d, $handler);
-            $builder->is_eq($mistakes, 0,"$cname Info has no spelling errors");
-        } else {
-            $builder->ok(0, "$cname has an Info field");
-        }
         foreach my $tpara (@tagpara) {
             my $tag = $tpara->{'tag'}//'';
             my $severity = $tpara->{'severity'}//'';
-            my $certainty = $tpara->{'certainty'}//'';
             my $info = $tpara->{'info'} // '';
             my (@htmltags, %seen);
 
@@ -256,13 +218,10 @@ sub test_check_desc {
             $builder->cmp_ok(length $tag, '<=', 68, 'Tag is not too long')
               or $builder->diag("$cname: $tag\n");
 
-            # Severity / Certainty
+            # Severity
             $builder->ok($severity && exists $severities{$severity},
                 'Tag has valid severity')
               or $builder->diag("$cname: $tag severity: $severity\n");
-            $builder->ok($certainty && exists $certainties{$certainty},
-                'Tag has valid certainty')
-              or $builder->diag("$cname: $tag certainty: $certainty\n");
 
             # Info
             my $mistakes = 0;
@@ -355,15 +314,17 @@ sub test_load_profiles {
 
     $opt{'wanted'} = sub {
         my $profname = $File::Find::name;
-        my ($err, $prof);
 
-        return unless $profname =~ s/\.profile$//o;
+        return
+          unless $profname =~ s/\.profile$//o;
         $profname =~ s,^$sre,,;
 
-        eval {$prof = Lintian::Profile->new($profname, \@inc);};
-        $err = $@;
+        my $profile = Lintian::Profile->new;
 
-        $builder->ok($prof, "$profname is loadable.")
+        eval {$profile->load($profname, \@inc);};
+        my $err = $@;
+
+        $builder->ok($profile, "$profname is loadable.")
           or $builder->diag("Load error: $err\n");
     };
 
@@ -433,16 +394,20 @@ sub test_load_checks {
         my $find_opt = {'want-check-name' => 1,};
         $find_opt->{'filter'} = $opts->{'filter'} if exists $opts->{'filter'};
         @checknames = _find_check($find_opt, $dir);
-        $builder->cmp_ok(scalar @checknames, '>', 0, 'Found checks to test');
     } else {
         $builder->skip('Given an explicit list of checks');
     }
 
+    $builder->skip('No desc files found')
+      unless @checknames;
+
     load_profile_for_test();
 
     foreach my $checkname (@checknames) {
-        my $cs;
-        eval {$cs = Lintian::CheckScript->new($dir, $checkname);};
+        my $cs = Lintian::Check::Info->new;
+        $cs->basedir($dir);
+        $cs->name($checkname);
+        eval {$cs->load;};
         if (my $err = $@) {
             $err =~ s/ at .*? line \d+\s*\n//;
             $builder->ok(0, "Cannot parse ${checkname}.desc");
@@ -452,7 +417,7 @@ sub test_load_checks {
         }
         my $cname = $cs->name;
         my $ppkg = $cname;
-        my $path = $cs->script_path;
+        my $path = $cs->path;
         my $err;
         my $rs_ref = 'MISSING';
 
@@ -467,16 +432,21 @@ sub test_load_checks {
         $ppkg =~ s,[-.],_,go;
         $ppkg =~ s,/,::,go;
         $ppkg = "Lintian::$ppkg";
-        eval {
-            # minimal "no strict refs" scope.
-            no strict 'refs';
-            $rs_ref = 'PRESENT'
-              if defined &{$ppkg . '::run'};
-        };
-        $err = $@//'';
-        if (!$builder->is_eq($rs_ref, 'PRESENT', "Check $cname has runsub")) {
+
+        if ($ppkg->can('run') && !$ppkg->DOES('Lintian::Check')) {
+            $builder->diag(
+                "Warning: check $ppkg uses old entry point ::run\n");
+        }
+
+        # setup and breakdown should only be used together with files
+        my $has_entrypoint = any { $ppkg->can($_) }
+        (
+            'source', 'binary', 'udeb', 'installable',
+            'changes', 'always', 'files'
+        );
+
+        if (!$builder->ok($has_entrypoint, "Check $cname has entry point")) {
             $builder->diag("Expected package name is $ppkg\n");
-            $builder->diag("Error: $err\n") if $err;
         }
     }
     return;
@@ -541,9 +511,7 @@ Consider the following example:
  $tagname = 'my-other-tag' if $condition;
 
 In this example, this test would conclude that 'my-tag' and
-'my-other-tag' are both implemented.  Which is good when $tagname is
-eventually passed to L<tag|Lintian::Tags/tag>, and a false-negative
-otherwise.
+'my-other-tag' are both implemented.
 
 Comment lines are I<not> ignored, so comments can be used as an
 alternative to the exclude-pattern (above).
@@ -566,7 +534,6 @@ sub test_tags_implemented {
         my $find_opt = {'want-check-name' => 1,};
         $find_opt->{'filter'} = $opts->{'filter'} if exists $opts->{'filter'};
         @checknames = _find_check($find_opt, $dir);
-        $builder->cmp_ok(scalar @checknames, '>', 0, 'Found checks to test');
     } else {
         $builder->skip('Given an explicit list of checks');
     }
@@ -580,8 +547,11 @@ sub test_tags_implemented {
     }
 
     foreach my $checkname (@checknames) {
-        my ($cs, @tags, $codestr, @missing);
-        eval {$cs = Lintian::CheckScript->new($dir, $checkname);};
+        my (@tags, $codestr, @missing);
+        my $cs = Lintian::Check::Info->new;
+        $cs->basedir($dir);
+        $cs->name($checkname);
+        eval {$cs->load;};
         if (my $err = $@) {
             $err =~ s/ at .*? line \d+\s*\n//;
             $builder->ok(0, "Cannot parse ${checkname}.desc");
@@ -589,7 +559,7 @@ sub test_tags_implemented {
             next;
         }
         my $cname = $cs->name;
-        my $check = $cs->script_path;
+        my $check = $cs->path;
 
         @tags = $cs->tags unless defined $pattern;
         @tags = grep { !m/$pattern/ } $cs->tags
@@ -650,12 +620,14 @@ sub load_profile_for_test {
      # do just fine...
     $profname ||= 'debian/main';
 
-    unless (@inc) {
-        push @inc, $ENV{'LINTIAN_TEST_ROOT'} if $ENV{'LINTIAN_TEST_ROOT'};
-        push @inc, '/usr/share/lintian' if -d '/usr/share/lintian';
-    }
+    my $location
+      = $ENV{'LINTIAN_ROOT'} ? $ENV{'LINTIAN_ROOT'} : '/usr/share/lintian';
+    push(@inc, $location)
+      unless @inc;
+
+    $PROFILE = Lintian::Profile->new;
+    $PROFILE->load($profname, \@inc);
 
-    $PROFILE = Lintian::Profile->new($profname, \@inc);
     Lintian::Data->set_vendor($PROFILE);
     $ENV{'LINTIAN_HELPER_DIRS'} = join(':', map { "$_/helpers" } @inc);
     $ENV{'LINTIAN_INCLUDE_DIRS'} = join(':', @inc);
@@ -808,3 +780,9 @@ files/symlinks beneath it.
 =cut
 
 1;
+
+# Local Variables:
+# indent-tabs-mode: nil
+# cperl-indent-level: 4
+# End:
+# vim: syntax=perl sw=4 sts=4 sr et
