@@ -33,8 +33,9 @@ Various hook routines for the test runners.
 
 =cut
 
-use strict;
+use v5.20;
 use warnings;
+use utf8;
 use autodie;
 
 use Exporter qw(import);
@@ -57,8 +58,6 @@ use File::Find::Rule;
 use File::Path;
 use File::stat;
 use Path::Tiny;
-use File::Temp qw(tempfile);
-use Text::Template;
 
 use constant NEWLINE => qq{\n};
 use constant SPACE => q{ };
@@ -159,7 +158,8 @@ sub find_missing_prerequisites {
       || $testcase->{test_conflicts};
 
     # create a temporary file
-    my ($temphandle, $tempfile)= tempfile('bd-test-XXXXXXXXX', TMPDIR => 1);
+    my $temp = Path::Tiny->tempfile(
+        TEMPLATE => 'lintian-test-build-depends-XXXXXXXXX');
     my @lines;
 
     # dpkg-checkbuilddeps requires a Source: field
@@ -178,17 +178,13 @@ sub find_missing_prerequisites {
     push(@lines, "Build-Conflicts: $build_conflicts")
       if length $build_conflicts;
 
-    say {$temphandle} join(NEWLINE, @lines);
-    close($temphandle) or carp "Could not close temporary file: $!";
+    $temp->spew(join(NEWLINE, @lines) . NEWLINE);
 
     # run dpkg-checkbuilddeps
-    my $command = "dpkg-checkbuilddeps $tempfile";
+    my $command = "dpkg-checkbuilddeps $temp";
     my ($missing, $status) = capture_merged { system($command); };
     $status = ($status >> 8) & 255;
 
-    # delete temporary file
-    unlink($tempfile);
-
     die "$command failed: $missing" if !$status && length $missing;
 
     # parse for missing prerequisites
@@ -275,3 +271,8 @@ sub run_lintian {
 
 1;
 
+# Local Variables:
+# indent-tabs-mode: nil
+# cperl-indent-level: 4
+# End:
+# vim: syntax=perl sw=4 sts=4 sr et
